/**************************************************************************
*                                                                                         
*  mcfcounter.h - counter specific defines and functions for the Coldfire GPT                                               
*                                                                                         
*  (C) Copyright 2005, EMAC.Inc www.emacinc.com                                           
*      written by              NZG ngustavson@emacinc.com                                                                                                                   
**************************************************************************/

#ifndef MCFCOUNTER_H_
#define MCFCOUNTER_H_

/*******************************************************************************
 * data types
 */
typedef volatile u16 mcfcounter;

typedef struct mcfcount_device{
	int id;
	char name[10];
	int status;
	mcfcounter *count;
	struct fasync_struct *overflow;
}mcfcount_device;

/*******************************************************************************
 * prototypes and exported functions
 */
irqreturn_t cfgptic_isr(int irq, void *dev_id, struct pt_regs *regs);
irqreturn_t pulse_overflow_isr(int irq, void *dev_id, struct pt_regs *regs);
mcfcount_device *ic_count_init(int id);
mcfcount_device *pulse_accum_init(int id);

/*******************************************************************************
 * low-level inline functions
 */
 static inline void PULSE_ACCA_ENABLE(void)
	{ 	GPTPACTLA |=PEDGE;
		GPTPACTLA |=PAE;
		ICR_PULSE_OVERFLOWA = (PULSE_LEVEL<<3) + PULSE_PRIORITY;
		SDEBUG("	GPTPACTLA = 0x%x\n",GPTPACTLA);
		SDEBUG("	ICR_PULSE_OVERFLOWA  = 0x%x\n",ICR_PULSE_OVERFLOWA );}

static inline void PULSE_ACCA_DISABLE(void)
	{ 	GPTPACTLA &=~PAE;
		SDEBUG("	GPTPACTLA = 0x%x\n",GPTPACTLA);}

static inline void PULSE_ACCA_OVERFLOW_ISR_ENABLE(void)
	{ 	GPTPACTLA |= PAOVI;
		IRQMRH&=~(1<<(PULSE_OVERFLOWA_IRQ - 32));
		SDEBUG("	GPTPACTLA= 0x%x\n",GPTPACTLA);
		SDEBUG("	IRQMRH = 0x%lx\n",IRQMRH);
		}
		
static inline void PULSE_OVERFLOW_IRQ_CLEAR(int id)	
	{	GPTPAFLGA = PAOVF;
		SDEBUG("	GPTFLG1A = 0x%x\n",GPTFLG1A);
		}
		
static inline void IC_IRQ_ENABLE(int id)
	{ 	GPTIEA |= (1<<(id - ICA_OFFSET));
		IRQMRH&=~(1<<(id - ICA_OFFSET + GPTA_ISR_BASE - 32));
		SDEBUG("	GPTIEA = 0x%x\n",GPTIEA);
		SDEBUG("	IRQMRH = 0x%lx\n",IRQMRH);
		SDEBUG_SHOW_STATUS();
		CFGPT_IRQ_CLEAR(id);
		SDEBUG_SHOW_STATUS();	
		}
	
static inline void IC_IRQ_DISABLE(int id)
	{ 	GPTIEA &=~( 1<<(id - ICA_OFFSET));
		SDEBUG("	GPTIEA = 0x%x\n",GPTIEA);
		}

static inline void IC_DDRIN(int id)
	{ 	GPTDDRA	&=~( 1<<(id - ICA_OFFSET));
		SDEBUG("	GPTDDRA = 0x%x\n",GPTDDRA);
		}

static inline void IC_ENABLE(int id)
	{	OC_DISABLE(id);
		GPTIOSA&=~(1<<((id-ICA_OFFSET)));
		IC_DDRIN(id);
		GPTCTL2A&=~(ANYE<<((id-ICA_OFFSET)*2));
		GPTCTL2A|=RISEE<<((id-ICA_OFFSET)*2);
		ICR_GPTA_ARRAY[id-ICA_OFFSET] = (IC_LEVEL<<3) + IC_PRIORITY_BASE+id;
		SDEBUG("	GPTCTL2A = 0x%x\n"	,GPTCTL2A);
		SDEBUG("	GPTIOSA = 0x%x\n"	,GPTIOSA);
		SDEBUG("	ICR_GPTA_ARRAY[%u] = 0x%x\n",
			id ,ICR_GPTA_ARRAY[id-ICA_OFFSET]);
		SDEBUG_SHOW_STATUS();
		}

/*******************************************************************************
 * cfcount_disable
 * Disables the counter passed to it.
 * Typically this is used to stop a counter, which can then be restarted later
 * @param *dev A pointer to a mcfcount_device structure
 */
static inline void cfcount_disable(mcfcount_device *dev){
		switch(dev->id){
		
		case PULSE_ACCA:
			PULSE_ACCA_DISABLE();
		break;
	
		default:
			IC_IRQ_DISABLE(dev->id);
		}
	dev->status&=~ENABLED;
}

/*******************************************************************************
 * cfcount_enable
 * Enables the counter passed to it.
 * An enabled counter will run if it is also INITIALIZED
 * @param *dev A pointer to a mcfcount_device structure
 * @return pointer to the appropriate initialized mcfcount_device structure
 */
static inline void cfcount_enable(mcfcount_device *dev){
		switch(dev->id)	{	
		
		case PULSE_ACCA:
			 PULSE_ACCA_ENABLE();
		break;
		
		default:
			 IC_IRQ_ENABLE(dev->id);
		}
	dev->status|=ENABLED;	
}	
	
/*******************************************************************************
 * mcfcounter_init
 * retrieve and initialize an appropriate mcfcount_device structure for 
 * the given id.
 * @parm id - id number of the device
 * @return pointer to the appropriate initialized mcfcount_device structure
 */	
static inline mcfcount_device *mcfcounter_init(int id){
	mcfcount_device *dev;
		
	GENDEBUG("mcfcounter_init %u\n",id);	
		
	switch(id){
		case PULSE_ACCA:
			dev = pulse_accum_init(id);
		break;

		default:
			dev = ic_count_init(id);
	}
	return dev;		
}

#endif /*MCFCOUNTER_H_*/
