/**************************************************************************
*                                                                                         
*  cfcount.c - module for utilizing GPT module of the MCF5282                                               
*                                                                                         
*  (C) Copyright 2005, EMAC.Inc www.emacinc.com                                           
*      written by              NZG ngustavson@emacinc.com                                 
*                                                                                         
**************************************************************************/
#include <asm/uaccess.h>
#include <linux/cdev.h>
#include <linux/fs.h>
#include <linux/counter.h>
#include <linux/pwm.h>
#include "cfgpt.h"
#include "cfcounter.h"
#include "cfpwm.h"

static struct cdev *cfgpt_cdev;

mcfcounter soft_count[SOFTCOUNT_NUM];
mcfcount_device counter[TOTAL_COUNTER_DEVICES]; 
mcfpwm_device pwm[TOTAL_PWM_DEVICES]; 

irqreturn_t cfgptic_isr(int irq, void *dev_id, struct pt_regs *regs){
	 mcfcount_device *dev = dev_id;
	 
	 IRQDEBUG("cfgptic_isr - %u\n",dev->id);
	 IRQDEBUG("*dev->count++=%x\n",*dev->count);
	 *dev->count =*dev->count+1 ;
	 
	if((!(*dev->count))&&(dev->overflow))
			kill_fasync(&dev->overflow, SIGIO, POLL_IN);//notify async queue1`
			
	CFGPT_IRQ_CLEAR(dev->id);			
return IRQ_RETVAL(1);
}

irqreturn_t pulse_overflow_isr(int irq, void *dev_id, struct pt_regs *regs){
	 mcfcount_device*dev = dev_id;
	 
	 IRQDEBUG("pulse_overflow_isr\n");
	 	 
	if(dev->overflow)
		kill_fasync(&dev->overflow, SIGIO, POLL_IN);//notify async queue
		
	PULSE_OVERFLOW_IRQ_CLEAR(dev->id);		
return IRQ_RETVAL(1);
}

 mcfcount_device *ic_count_init(int id){
	
	 mcfcount_device *dev = &counter[id];
	
	if(!(dev->status&INITIALIZED)){
		dev->id = id;
		dev->count =&soft_count[id];
		*dev->count = 0;
		GENDEBUG("dev->count=%lx\n",(unsigned long)dev->count);
		sprintf(dev->name,"GPTA IC%u",id);
		IC_ENABLE(id);
		request_irq(GPTA_ISR_BASE+id+ICR0_BASE,
		 	cfgptic_isr, SA_INTERRUPT, dev->name, dev);
		dev->status = INITIALIZED;
	GENDEBUG("%s initialized\n",dev->name);
	}
	
	return dev;
}

 mcfcount_device *pulse_accum_init(int id){
	
	 mcfcount_device *dev = &counter[id];
	
	if(!(dev->status&INITIALIZED)){
		dev->id = id;
		dev->count = PULSE_ACCA_ADDRESS;
		*dev->count = 0;
		sprintf(dev->name,"GPTA PA");
		PULSE_ACCA_OVERFLOW_ISR_ENABLE();
		request_irq(PULSE_OVERFLOWA_IRQ+ICR0_BASE, 
			pulse_overflow_isr, SA_INTERRUPT, dev->name, dev);
		dev->status = INITIALIZED;
	}
	
	return dev;
}

mcfpwm_device *mcfpwm_init(int id){
	
	 mcfpwm_device *dev = &pwm[id];
	
	GENDEBUG("mcfpwm_init\n");
	
	if(!(dev->status&INITIALIZED)){
		GENDEBUG("initializing structure %x\n",id);
		dev->id = id;
		sprintf(dev->name,"GPTA PWM%u",id);
		IOCPWM_CONFIG(id);
		dev->status = INITIALIZED;
	}
	
	return dev;
}

/**
 * cfcount_ioctl
 * System Call from user space to control a GPT counter.
 * Called by the kernel in response to a ioctl call on a gpt counter device
 */
static int cfcount_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
		     unsigned long arg){
	
	 mcfcount_device  *dev = filp->private_data;
	
	switch(cmd){
		case ENABLE_COUNT:	/*Enable  Counter*/
			GENDEBUG("enable count %u\n",dev->id);
			cfcount_enable(dev);			
		break;
			
		case DISABLE_COUNT:	/* Disable Counter	*/
			GENDEBUG("disable count %u\n",dev->id);
			cfcount_disable(dev);				
		break;
		
		case WRITE_COUNT:	/* Write a value to a counter device*/		
			GENDEBUG("write count %u\n",dev->id);	
			if (copy_from_user((void *)dev->count, (void __user *)arg,
				   sizeof(mcfcounter)))
			return -EFAULT;	
		break;
				
		case READ_COUNT:	/* Read Counter	*/
		GENDEBUG("read count %u\n",dev->id);
		GENDEBUG("dev->count=%lx\n",(unsigned long)dev->count);
		GENDEBUG("*dev->count=0x%x\n",*dev->count);
		SDEBUG_SHOW_STATUS();
		if(copy_to_user((void __user *)arg, (void *)dev->count, sizeof (mcfcounter)))
			return -EFAULT;
		return 0;

		case READ_COUNT_TS:{	
			tscount count;
			unsigned long flags=0;
			local_irq_save(flags);
		    count.count = *dev->count;
		    count.ts = rdtsc();
			local_irq_restore(flags);
			
			GENDEBUG("read %x from device %u\n",count, dev->id);
			if(copy_to_user((void __user *)arg, (void *)&count, sizeof (tscount)))
				return -EFAULT;
			return 0;
		}


		default:
			return -EINVAL;
	}    
	return 0;
}

/**
 * cfpwm_ioctl
 * System Call from user space to control a GPT pwm.
 * Called by the kernel in response to a ioctl call on a gpt counter device
 */
static int cfpwm_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
		     unsigned long arg){
	
	 mcfpwm_device  *dev = filp->private_data;
	
	switch(cmd){		
		case SET_FREQ:{	/* Set global counter frequency*/		
			if(GPTA_SETFREQ(arg)<0){
				GENDEBUG("invalid frequency\n");
				return -EFAULT;
			}
		GENDEBUG("set master counter freq to %lu\n",arg);	
		}	
		break;
				
		case SET_DUTY:	/* Set individual duty cycle, note
		 that this must be done after the frequency is set to work correctly	*/
		GENDEBUG("set duty cycle of %s to %lx\n",dev->name,arg);
		dev->duty = arg;
		SetIOCA(dev->id,dev->duty);
		break;

		default:
			return -EINVAL;
	}    
	SDEBUG_SHOW_STATUS();
	SDEBUG_SHOW_PWMREGS();
	return 0;
}


static int cfcount_fasync(int fd, struct file *filp, int mode){
	 mcfcount_device  *dev = filp->private_data;
	return(fasync_helper(fd, filp, mode, &dev->overflow)); 
}

static int cfgpt_release(struct inode *inode, struct file *filp){
	filp->private_data = NULL;
	return 0;	
}

struct file_operations mcfcount_fops={
  ioctl:   	cfcount_ioctl,
  fasync:  cfcount_fasync,
  release: 	cfgpt_release,
};

struct file_operations mcfpwm_fops={
  ioctl:   	cfpwm_ioctl,
  release: 	cfgpt_release,
};


static int cfgpt_open (struct inode *inode, struct file *filp){
	int minor = MINOR(inode->i_rdev);
	int type = minor>>4;
	int num = minor&0x0f;		
	
	GENDEBUG("initializing %s structure %x \n",(type==COUNTER_TYPE)?"counter":"pwm",num);
	
	switch(type){
		case COUNTER_TYPE:
			filp->private_data = mcfcounter_init(num);
			filp->f_op = &mcfcount_fops;
		break;
		
		case PWM_TYPE:
			filp->private_data = mcfpwm_init(num);
			filp->f_op = &mcfpwm_fops;
		break;
	}
	
	return 0;
}

struct file_operations cfgpt_fops={
 open:    	cfgpt_open,
  release: 	cfgpt_release,
};

static inline void register_as_char(void){
	dev_t dev = MKDEV(GPTMAJOR ,FIRSTGPTMINOR);
	int result;
	
	cfgpt_cdev = cdev_alloc();
	
	if((result=register_chrdev_region(dev, GPTMINORNUM,GPTNAME))<0)
		printk(GPTNAME " driver failed to register Major Number"); 

	cfgpt_cdev->ops = &cfgpt_fops;
	cfgpt_cdev->owner = THIS_MODULE;
	cdev_add(cfgpt_cdev,dev,GPTMINORNUM);	
	
	printk(GPTNAME" driver registered\n");
	//printk("Major 0x%x Minor 0x%x\n", GPTMAJOR,FIRSTGPTMINOR);
	}

static int __init cfgpt_init(void){
	printk( GPTNAME " driver version " GPT_DRIVER_V " (c) " __DATE__  "\n");
	printk("EMAC.Inc\n");
	 register_as_char();
	GPT_ENABLE();
	return 0;
}

static inline void unregister_as_char(void){
	dev_t devno = MKDEV(GPTMAJOR ,FIRSTGPTMINOR);
	unregister_chrdev_region(devno, GPTMINORNUM);
	cdev_del(cfgpt_cdev);
}

static void __exit cfgpt_exit (void){
	//unregister_chrdev(DSMajor,DSNAME);
	//misc_deregister(&gpt_dev);
	unregister_as_char();
	printk(GPTNAME" driver unloaded\n");
}

module_init(cfgpt_init);
module_exit(cfgpt_exit);




		
