/**************************************************************************
*                                                                                         
*  cfgpt.h - general purpose timer of the SoM-5282EM                                               
*                                                                                         
*  (C) Copyright 2005, EMAC.Inc www.emacinc.com                                           
*      written by              NZG ngustavson@emacinc.com                                                                                                                   
**************************************************************************/
#ifndef _CFGPT_H_
#define _CFGPT_H_

#include <asm/types.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/config.h>

#define COLDFIRE_INTERNAL_DEV_MAJORA 102

#define GPTNAME 			"cfgpt"
#define GPT_DRIVER_V 	"1.2" 

/**
 * device types
 */
#define COUNTER_TYPE						0
#define PWM_TYPE								1

#define ICA_OFFSET 							0
#define SOFTCOUNT_NUM					4
#define PA_NUM									1
#define TOTAL_COUNTER_DEVICES	(SOFTCOUNT_NUM+PA_NUM)
#define  PULSE_ACCA 						SOFTCOUNT_NUM	

#define PWMA_OFFSET						PULSE_ACCA+1
#define PWM_NUM								3
#define TOTAL_PWM_DEVICES			PWM_NUM

#define GPTMAJOR 			 					COLDFIRE_INTERNAL_DEV_MAJORA
#define FIRSTGPTMINOR  					0
//#define GPTMINORNUM   					(TOTAL_COUNTER_DEVICES+TOTAL_PWM_DEVICES)
#define GPTMINORNUM   					0x20


/*******************************************************************************
 * register definitions
 */
#include <asm/coldfire.h>             
 #include <asm/m528xsim.h>
 
 #define IRQMRH (*((volatile unsigned long *) (MCF_IPSBAR+MCFICM_INTC0+MCFINTC_IMRH)))
 #define IRQMRL (*((volatile unsigned long *) (MCF_IPSBAR+MCFICM_INTC0+MCFINTC_IMRL)))
 
#define GPTA_BASE (MCF_IPSBAR+0x1A0000)
#define GPTB_BASE (MCF_IPSBAR+0x1B0000)
 
 #define GPTIOSA			( *(volatile u8 *)(GPTA_BASE + 0x00))
 
 #define GPCFORCA	( *(volatile u8 *)(GPTA_BASE + 0x01))
 
 #define GPTOC3MA		( *(volatile u8 *)(GPTA_BASE + 0x02))
 
 #define GPTOC3DA		( *(volatile u8 *)(GPTA_BASE + 0x03))
 
 #define GPTSCR1A		( *(volatile u8 *)(GPTA_BASE + 0x06))
 	#define GPTEN			0x80
 	#define TFFCA			0x10
 
#define GPTTOVA			( *(volatile u8 *)(GPTA_BASE + 0x08))

#define GPTCTL1A		( *(volatile u8 *)(GPTA_BASE + 0x09))
	#define OCNO			0x00
	#define OCTO			0x01
	#define OCCL			0x02
	#define OCST			0x03

#define GPTCTL2A		( *(volatile u8 *)(GPTA_BASE + 0x0B))
	#define NOE				0x00
	#define RISEE			0x01
	#define FALLE			0x02
	#define ANYE			0x03

#define GPTIEA				( *(volatile u8 *)(GPTA_BASE + 0x0C))

#define GPTSCR2A		( *(volatile u8 *)(GPTA_BASE + 0x0D))
	#define TCRE			8
	#define PRMAX			7

#define GPTFLG1A		( *(volatile u8 *)(GPTA_BASE + 0x0E))	

#define GPTFLG2A		( *(volatile u8 *)(GPTA_BASE + 0x0F))	

#define GPTCA				( (volatile u16 *)(GPTA_BASE + 0x10))
	#define GPTMAX		0xffff

#define GPTPACTLA		( *(volatile u8 *)(GPTA_BASE + 0x18))
	#define PAI				0x01
	#define PAOVI			0x02	
	#define PEDGE			0x10
	#define PAE				0x40

#define GPTPAFLGA		 (*(volatile u8 *)(GPTA_BASE + 0x19))
	#define PAOVF			0x02

#define GPTPACNTA		 (*(volatile u16 *)(GPTA_BASE + 0x1A))

#define GPTDDRA		( *(volatile u8 *)(GPTA_BASE + 0x1E))

#define ICR0_BASE							64
#define PULSE_OVERFLOWA_IRQ	43
#define GPTA_ISR_BASE 					44

#define IC_LEVEL								4
#define IC_PRIORITY_BASE 			0	

#define PULSE_LEVEL 		4
#define PULSE_PRIORITY 	PULSE_ACCA

#define ICR_PULSE_OVERFLOWA (*(volatile u8 *) (MCF_IPSBAR + MCFICM_INTC0 + MCFINTC_ICR0 + PULSE_OVERFLOWA_IRQ))              

#define ICR_GPTA_ARRAY ((volatile u8 *) (MCF_IPSBAR + MCFICM_INTC0 + MCFINTC_ICR0 + GPTA_ISR_BASE))
                              
#define PULSE_ACCA_ADDRESS 		&GPTPACNTA

/* software bitmasks*/
#define INITIALIZED								1
#define ENABLED								2

/*******************************************************************************
 * DEBUG Macros
 */
//#define  DEBUG_IRQS
//#define  DEBUG_GEN
//#define  DEBUG_LOW

#ifdef DEBUG_IRQS
#define IRQDEBUG(fmt, args...) printk(fmt, ## args)
#else
#define IRQDEBUG(fmt, args...)
#endif

#ifdef DEBUG_GEN
#define GENDEBUG(fmt, args...) printk(fmt, ## args)
#else
#define GENDEBUG(fmt, args...) 
#endif

#ifdef DEBUG_LOW
#define SDEBUG(fmt, args...) printk(fmt, ## args)
#else
#define SDEBUG(fmt, args...) 
#endif

/*******************************************************************************
 * low-level inline functions
 */

static inline void CFGPT_IRQ_CLEAR(int id)
	{	GPTFLG1A =(1<<(id - ICA_OFFSET)); 
		SDEBUG("	GPTFLG1A = 0x%x\n",GPTFLG1A);
		}

static inline void SDEBUG_SHOW_STATUS(void)
	{	SDEBUG("	GPTSCR1A = 0x%x\n",GPTSCR1A	);
		SDEBUG("	GPTFLG1A = 0x%x\n",GPTFLG1A	);
		SDEBUG("	GPTFLG2A = 0x%x\n",GPTFLG2A	);
		}

static inline void OC_DISABLE(int id)
	{	GPTCTL1A&=~(OCST<<((id-ICA_OFFSET)*2));
		SDEBUG("	GPTCTL1A= 0x%x\n"	,GPTCTL1A);
		}

static inline void GPT_ENABLE(void)
	{	GPTSCR1A|=GPTEN;
		//GPTSCR1A|=GPTEN|TFFCA;
		//GPTSCR2A=7;	
		SDEBUG("	GPTSCR1A = 0x%x\n",GPTSCR1A);
		SDEBUG("	GPTSCR2A = 0x%x\n",GPTSCR2A);
		}

static inline void GPT_DISABLE(void)
	{	GPTSCR1A&=~GPTEN;
		//GPTSCR1A|=GPTEN|TFFCA;
		//GPTSCR2A=7;	
		SDEBUG("	GPTSCR1A = 0x%x\n",GPTSCR1A);
		SDEBUG("	GPTSCR2A = 0x%x\n",GPTSCR2A);
		}

/**
 * Use GPT3 and the prescaler to cause precise reloads of the timer 
 * and generate a specific frequency.
 */		
static inline int GPTA_SETFREQ(unsigned long freq)
	{//need to set prescaler here appropriately
 	unsigned long afreq = MCF_CLK/2;
 	unsigned long reload = afreq/freq;
 	int pr;
 	
 	GENDEBUG("setting master timerA frequency\n");
 	
 	if(reload==0){
 		GENDEBUG("requested frequency to high\n");
 		return -1;//requested frequency too high
 	}
 	
 	for(pr=0;pr<=PRMAX;pr++){
 		 	GENDEBUG("trying  afreq=%lx reload=%lx \n",afreq,reload);
 			if(reload<=GPTMAX){
 				GPTCA[3] = reload;
 				GPTSCR2A = TCRE | pr; 
 				GPTIOSA|=8;//SET GPT3 to OC	
 				GENDEBUG("set pr =%x, GPTCA3=%x \n",pr,GPTCA[3]);
				return 1;
 			}
 			afreq/=2;
 			reload = afreq/freq;
 	}
 	GENDEBUG("requested frequency to low\n");
	return -2; //requested frequency too low;
}


#endif//#ifndef _CFGPT_H_
