/**************************************************************************
*                                                                                         
*  cfpwm.h - pulse width modulation utilizing output compare pins of the SoM-5282EM                                               
*                                                                                         
*  (C) Copyright 2005, EMAC.Inc www.emacinc.com                                           
*        support@emacinc.com                                                                                                                   
**************************************************************************/
#ifndef _CFPWM_H_
#define _CFPWM_H_

typedef struct mcfpwm_device{
	int id;
	char name[10];
	int status;
	unsigned int  duty;
}mcfpwm_device;

/**
 * Configure timer port 0-2  of GPTA to be an IOC.
 * the frequency will be the same as the reload frequency of timer.
 * Note that different timer frequencies will have different duty cycle resolutions.
 * The best resolutions will be at PRESCALEDCLOCK/GPTCAMAX frequencies, since the duty cycle OC
 * will be able to pick any value between 0-GPTCAMAX, a value of about 500Hz  to gives a good duty cycle, using a reload
 * value close to 0xffff. 
 * resolution
 * Also note that this function only works correctly when the GPTCA3 OC is in the correct state before it is called.
 */
static inline void SetIOCA(int ioc, unsigned int duty){
			unsigned long scaled_duty=0;
			if(GPTCA[3])
				scaled_duty= duty/(UINT_MAX/GPTCA[3]);//PWM fixed point approximation
					 	
			GENDEBUG("setting duty to %x, scaled to %lx\n",duty,scaled_duty);
			GPTCA[ioc] =scaled_duty;//set cutoff timeout
			
			if(scaled_duty)
				GPTOC3DA|=(1<<((ioc-ICA_OFFSET)));//set pin high on overflow event
			else	 	
				GPTOC3DA&=~(1<<((ioc-ICA_OFFSET)));//set pin low on overflow event (duty cycle 0)
}

/**
 * this should probably return a device node.
 */
static inline void  IOCPWM_CONFIG(int ioc){
			GPTIOSA|=(1<<((ioc-ICA_OFFSET)));//enable IOC functionality
			GPTTOVA&=~(1<<((ioc-ICA_OFFSET)));//disable toggle on overflow
			GPTOC3MA|=(1<<((ioc-ICA_OFFSET)));//unmask bit for channel 3 overflow
			//GPTOC3DA|=(1<<((ioc-ICA_OFFSET)));//set pin high on overflow event
			GPTCTL1A&=~(3<<((ioc-ICA_OFFSET)*2));//clear previous OC setting
			GPTCTL1A|=(OCCL<<((ioc-ICA_OFFSET)*2));//OC clears
			GPTCA[ioc] = 0xffff; //default 100% duty cycle
			GPCFORCA|=(1<<((ioc-ICA_OFFSET)));//force an output compare (send it low)
}

static inline void SDEBUG_SHOW_PWMREGS(void){
		SDEBUG("	GPTIOSA = 0x%x\n",GPTIOSA);
		SDEBUG("	GPTTOVA = 0x%x\n",GPTTOVA);
		SDEBUG("	GPTCTL1A = 0x%x\n",GPTCTL1A);
		SDEBUG("	GPTCA0 = 0x%x\n",GPTCA[0]);
		SDEBUG("	GPTCA1 = 0x%x\n",GPTCA[1]);
		SDEBUG("	GPTCA2= 0x%x\n",GPTCA[2]);
		SDEBUG("	GPTCA3= 0x%x\n",GPTCA[3]);
		SDEBUG("	GPCFORCA= 0x%x\n",GPCFORCA);
}


#endif//#ifndef _CFPWM_H_
