/**************************************************************************
*                                                                                         
*  egpio.c - 2.6 character driver for EMAC GPIO modules
*  This driver was originally designed for the SoM-100ES MAX II CPLD, which contains
*  4 egpio modules by default, but is designed to be easily expandable to new designs .
*  Low level functions are declared as inlines in asm/egpio.h                                        
*                                                                                         
*  (C) Copyright 2005, EMAC.Inc www.emacinc.com                                           
*      written by              NZG ngustavson@emacinc.com                                 
*                                                                                         
**************************************************************************/
#include <linux/module.h>/* include file for kernel development*/
#include <linux/kernel.h>/* printk and others*/
#include <linux/fs.h>    /* the file structure */
#include <linux/ioport.h> /* inb, outb */
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/cdev.h>
#include <linux/egpio.h>
#include <asm/io.h> /*ioremap*/
#include <asm/uaccess.h>
#include <asm/gcfpld.h>

#define DRIVER_VERSION "1.0"
#define DRIVER_NAME "egpio"

MODULE_AUTHOR("EMAC.Inc-NZG");
MODULE_LICENSE("Dual BSD/GPL");

/**
 * First Minor number for GPIO devices
 */
 #define FIRSTGPIOMINOR 0
 #define GPIOMINORNUM 4

static struct cdev *gp100cdev;


static int GPIO100_Open (struct inode *inode, struct file *filp){           
return 0;//don't need anything special here
}

static int GPIO100_Close(struct inode *inode, struct file *filp){
return 0;//don't need anything special here
}

/**
 * GPIO100_IOCTL
 * used to read/write data to a register
 * command specifies read/write
 * arg specifies the index
 */
static int GPIO100_IOCTL(struct inode *inode, struct file *filp, unsigned int cmd,
                unsigned long arg){

unsigned int index = 2*iminor(inode - FIRSTGPIOMINOR);

switch(cmd){
	u8 data;

	case DATAREAD:
	data = GPIO_READ(index);
	put_user(data,(u8 *)arg);
	return 0;
	
	case DATAWRITE:
	get_user(data,(u8 *)arg);
	GPIO_WRITE(index,data);
	return 0;	
	
	case CONFIGREAD:
	data = GPIO_READ(index+1);
	put_user(data,(u8 *)arg);
	return 0;
	
	case CONFIGWRITE:
	get_user(data,(u8 *)arg);
	GPIO_WRITE(index+1,data);
	return 0;	
}

return -EINVAL;
}

struct file_operations GP100_Fops={
  .owner =    THIS_MODULE,
  .open   =    GPIO100_Open,
  .ioctl   =      GPIO100_IOCTL,
  .release =  GPIO100_Close,
};


static int gp100_init_module(void)
{
dev_t dev = MKDEV(GCFPLDMAJOR ,FIRSTGPIOMINOR);
int result;

gp100cdev = cdev_alloc();

printk(DRIVER_NAME" driver - version "DRIVER_VERSION" (c) " __DATE__  "\n");
printk(" N.Z. Gustavson (ngustavson@emacinc.com), EMAC.Inc\n");

result = register_chrdev_region(dev, GPIOMINORNUM,DRIVER_NAME);

if(result < 0){
	printk(DRIVER_NAME" driver failed to register Major Number"); 
	return result; 
	}

gp100cdev->ops = &GP100_Fops;
gp100cdev->owner = THIS_MODULE;

cdev_add(gp100cdev,dev,GPIOMINORNUM);

return 0;

}

/*cleanup_module
this routine is called when a module is unloaded with rmmod
*/
static void gp100_cleanup_module(void){
	dev_t devno = MKDEV(GCFPLDMAJOR ,FIRSTGPIOMINOR);
	unregister_chrdev_region(devno, GPIOMINORNUM);
	cdev_del(gp100cdev);
}


module_init(gp100_init_module);
module_exit(gp100_cleanup_module);
