/***************************************************************************
                          mcfqadc_char.c  -  character driver interface to the mcfqadc 
    					  (C) Copyright 2005, EMAC.Inc www.emacinc.com   
                             -------------------
    begin                : Fri Dec 23 2005
    email                : ngustavson@emacinc.com

 ***************************************************************************/

/***************************************************************************
 *                                                                         
 *   This program is free software; you can redistribute it and/or modify  
 *   it under the terms of the GNU General Public License as published by  
 *   the Free Software Foundation; either version 2 of the License, or     
 *   (at your option) any later version.                                   
 *                                                                         
 ***************************************************************************/
 #include <linux/module.h>/* include file for kernel development*/
#include <linux/kernel.h>/* printk and others*/
#include <linux/fs.h>    /* the file structure */
#include <linux/ioport.h> /* inb, outb */
#include <linux/delay.h>
#include <linux/interrupt.h>
//#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/cdev.h>
#include <asm/io.h> /*ioremap*/
#include <asm/uaccess.h>

#define CHARACTER_INTERFACE
#include <asm/mcfqadc.h>

#define DRIVER_VERSION "1.0"
#define DRIVER_NAME "mcfqadc"

MODULE_AUTHOR("EMAC.Inc-NZG");
MODULE_LICENSE("Dual BSD/GPL");

/**
 * First Minor number for GPIO devices
 */
 #define DAQMAJOR 101
 #define FIRSTQADCMINOR 2
 #define QADCMINORNUM 1

static struct cdev *qadccdev;


/**
 * QADC_IOCTL
 * QADC control and data exchange with user space via standard ioctl call
 * additional interfaces could be added here fairly easily.
 */
static int QADC_IOCTL(struct inode *inode, struct file *filp, unsigned int cmd,
                unsigned long arg){

qadc_dev *dev = filp->private_data;

QADCCDEBUG("QADC_IOCTL\n");

switch(cmd){
	case READALL:{
	u16 *buffer = QADC_Read_All(dev);
	int length = NUMCHANNELS*(sizeof(u16));
	if(access_ok(VERIFY_WRITE,(void *)arg,length)){
		QADCCDEBUG("copy to user %u bytes\n",length);
		copy_to_user((u16 *)arg,buffer,length);/*move data to user space*/
		return 0;
	}else{
		printk("couldn't write to user buffer\n");
		return -EINVAL;
	}}
	
	case GETCHANNELNUM:{
		int channum = NUMCHANNELS;
		QADCCDEBUG("GETCHANNELNUM\n");
		if(access_ok(VERIFY_WRITE,(void *)arg,(sizeof(int)))){
				QADCCDEBUG("access ok\n");
				put_user(channum,(int *)arg);
				return 0;
	}else{
		printk("couldn't write to user buffer\n");
		return -EINVAL;
	}}
	
}
return -EINVAL;
}

static int QADC_Open (struct inode *inode, struct file *filp){
	qadc_dev *dev =QADC_Device();
	  
	  QADCCDEBUG("QADC1290_Open\n");
	  
	  if(dev==NULL){
          		printk("couldn't create ltc device\n");
                return(-ENOMEM);
        }       
	filp->private_data = dev;        
return 0;//don't need anything special here
}

static int QADC_Close(struct inode *inode, struct file *filp){
	qadc_dev *dev = filp->private_data;
	filp->private_data  = QADC_Destroy_Device(dev);
return 0;//don't need anything special here
}


struct file_operations QADC1290_Fops={
  .owner =    THIS_MODULE,
  .open   =    QADC_Open,
  .ioctl   =      QADC_IOCTL,
  .release =  QADC_Close,
};


static int QADC_init_module(void){
dev_t dev = MKDEV(DAQMAJOR ,FIRSTQADCMINOR);
int result;

qadccdev = cdev_alloc();

printk(DRIVER_NAME" driver - version "DRIVER_VERSION" (c) " __DATE__  "\n");
printk(" N.Z. Gustavson (ngustavson@emacinc.com), EMAC.Inc\n");

result = register_chrdev_region(dev, QADCMINORNUM,DRIVER_NAME);

if(result < 0){
	printk(DRIVER_NAME" driver failed to register Major Number"); 
	return result; 
	}

qadccdev->ops = &QADC1290_Fops;
qadccdev->owner = THIS_MODULE;

cdev_add(qadccdev,dev,QADCMINORNUM);
return 0;
}

/*cleanup_module
this routine is called when a module is unloaded with rmmod
*/
static void QADC_cleanup_module(void){
	dev_t devno = MKDEV(DAQMAJOR ,FIRSTQADCMINOR);
	unregister_chrdev_region(devno, QADCMINORNUM);
	cdev_del(qadccdev);
}

module_init(QADC_init_module);
module_exit(QADC_cleanup_module);
