/**************************************************************************
*                                                                                         
*  rdtsc.c - 2.6 function calls for emulating a TSC with a Coldfire 32 bit DMA timer.                                      
*                                                                                         
*  (C) Copyright 2005, EMAC.Inc www.emacinc.com                                           
*      written by              NZG ngustavson@emacinc.com                                 
*                                                                                         
**************************************************************************/
#include <linux/module.h>/* include file for kernel development*/
#include <linux/kernel.h>/* printk and others*/
#include <linux/fs.h>    /* the file structure */
#include <linux/ioport.h> /* inb, outb */
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/cdev.h>
#include <linux/types.h>
#include <asm/io.h> /*ioremap*/
#include <asm/uaccess.h>
#include <asm/types.h>
#include <asm/mcfdmatimer.h>
#include <asm/timex.h>
#include <asm/system.h>

#define DRIVER_VERSION "1.0"
#define DRIVER_NAME "rdtsc"

MODULE_AUTHOR("EMAC.Inc-NZG");
MODULE_LICENSE("Dual BSD/GPL");

long long rt_time_rdtsc = 0;
static struct timer_list update_timer;
#define UPDATETTIMEOUT	HZ

//#define RDTSC_DEBUG

#ifdef RDTSC_DEBUG
#define RDTSCD(fmt, args...) printk(fmt, ## args)
#else
#define RDTSCD(fmt, args...)
#endif

/*****************************************************************************
 * start/restart update timer
 */
static inline void restart_timer(int timeout){
	update_timer.expires = jiffies + timeout;
	add_timer(&update_timer);
}

/**
 * periodically update the timestamp count, this really only needs to insure that rdtsc is called once before each
 * rollover (which takes over an hour)
 */
static void rdtsc_update_timer(void) {
	rdtsc();
	restart_timer(UPDATETTIMEOUT);
}

EXPORT_SYMBOL(rdtsc);
	
long long rdtsc(void) {
 unsigned  long flags=0;
  u32 diff=0;
  u32 count;
  local_irq_save(flags);
  count = DTCN3;
  diff = count - (u32)rt_time_rdtsc;
  rt_time_rdtsc += diff;
  local_irq_restore(flags);
  
  RDTSCD("RDTSC = 0x%llx\n",rt_time_rdtsc);
  RDTSCD("DTCN3 = 0x%x\n",count);
  RDTSCD("diff = 0x%x\n",diff);	
  	
  return rt_time_rdtsc;
};

/**
 * setup DMA time 4 to 1 tick/us
 */
static inline void	RDTSC_HARDWARE_INIT(void){
	DTMR3 = (((MCF_CLK/1000000) - 1)	<< 8) | MCFDMATIMER_TMR_CLK1 |
		MCFDMATIMER_TMR_RESTART | MCFDMATIMER_TMR_DISORI | MCFDMATIMER_TMR_ENABLE;
	RDTSCD("DTMR3 = 0x%x\n",DTMR3);	
}

/**
 * Initialize rdtsc module in the kernel
 */
static int rdtsc_init_module(void)
{
	printk(DRIVER_NAME" driver - version "DRIVER_VERSION" (c) " __DATE__  "\n");
	printk(" N.Z. Gustavson (ngustavson@emacinc.com), EMAC.Inc\n");

	printk("starting rdtsc timer\r\n");

	RDTSC_HARDWARE_INIT();

	init_timer(&update_timer);
	update_timer.function = (void *)rdtsc_update_timer;
	restart_timer(UPDATETTIMEOUT);

return 0;
}

/*cleanup_module
this routine is called when a module is unloaded with rmmod
*/
static void rdtsc_cleanup_module(void){
	del_timer(&update_timer);
}

module_init(rdtsc_init_module);
module_exit(rdtsc_cleanup_module);
