#ifndef CFCORE_WDT_H_
#define CFCORE_WDT_H_


/****************************************************************
 * Debug configuration
 */
//#define DEBUG_GEN

#ifdef DEBUG_GEN
#define GENDEBUG(fmt, args...) printk(fmt, ## args)
#else
#define GENDEBUG(fmt, args...) 
#endif

#ifdef CONFIG_M528x 

/****************************************************************
 * Core Watchdog control registers and inlines
 */
#define CWDT_BASE (MCF_IPSBAR)
 
 #define CWCR			( *(volatile u8 *)(CWDT_BASE + 0x11))
 	#define CWRI		0x40
 	#define CWE 		0x80
 	#define CWTA		0x04

 #define CWSR			( *(volatile u8 *)(CWDT_BASE + 0x13))
	#define M5282_CKICK_MAGICA	(0x55)
	#define M5282_CKICK_MAGICB	(0xAA)

#define	PORTE		( *(volatile u8 *)(MCF_IPSBAR + 0x100004))
#define DDRE		( *(volatile u8 *)(MCF_IPSBAR + 0x100018))
#define PEPAR		( *(volatile u16 *)(MCF_IPSBAR + 0x100052))
static inline void STATUS_LED_ENABLED(void){
	 DDRE|=0x01;
	 PEPAR&=0xfffc;
}
static inline void TOGGLE_STATUS_LED(void) {
	PORTE = ((PORTE&0x01))?(PORTE&0xfe):(PORTE|0x01);
	GENDEBUG("PORTE = %x\n",PORTE);
	GENDEBUG("DDRE = %x\n",DDRE);
	GENDEBUG("PEPAR = %x\n",PEPAR);
}

static inline void WATCHDOG_ENABLE(void)	{STATUS_LED_ENABLED();CWCR |= CWE;}

static inline void WDT_RESET_ON_TIMEOUT(void){CWCR |= CWRI;}

static inline void WDT_TA_BEFORE_RESET(void){CWCR |= CWTA;}



static inline void KICK_WATCHDOG(void){
	CWSR =  M5282_CKICK_MAGICA;
	CWSR =  M5282_CKICK_MAGICB;
	TOGGLE_STATUS_LED();
}

static inline void DUMP_WATCH_REGS(void){
	GENDEBUG("CWCR = %x\n",CWCR);
	GENDEBUG("CWSR = %x\n",CWSR);
}
/****************************************************************
 * Interrupt controller configuration
 */
#define MCFINT_CWDT			8              															/* Interrupt number for the Coldfire internal watchdog*/
#define MCFVEC_CWDT			(MCFINT_CWDT+MCFINT_VECBASE)              /* Interrupt Vector*/
#define ICR_CWDT 	* ((volatile u8 *) (MCF_IPSBAR + MCFICM_INTC0 + MCFINTC_ICR0 + MCFINT_CWDT))
#define MCFINT_WDT_LEVEL			7
#define MCFINT_WDT_PRIORITY		7
#define IRQMRL (*((volatile unsigned long *) (MCF_IPSBAR+MCFICM_INTC0+MCFINTC_IMRL)))

/**Set WDT watchdog interrupts level and priority*/
static inline void WDT_INTERRUPT_CONFIG(void){
	ICR_CWDT = (MCFINT_WDT_LEVEL<<3) + MCFINT_WDT_PRIORITY;
	IRQMRL&=~(1<<MCFINT_CWDT);
}

/****************************************************************
 * Coldfire Core Watchdog timeout configuration  
 */
 
 /**Input clock to the modules*/
#define MCLK (MCF_CLK/(2<<7)) 

/**
 * Available delays in seconds
 */
const int margin_set[] = { 
	1/MCLK,
	(2<<2)/MCLK, 
	(2<<4)/MCLK,
	(2<<6)/MCLK,
	(2<<10)/MCLK,
	(2<<14)/MCLK,
	(2<<18)/MCLK,
	(2<<22)/MCLK
}; 

/**size of the margin set*/
#define MARGINNUM (sizeof(margin_set)/sizeof(const int))

/**
 * set the prescaler to the Watchdog module, which in turn configures the timeout delay.
 */
static inline void WDT_SET_PRESCALE_EXP(u8 timeout){
		CWCR&=0xC7;
		CWCR|=((timeout&0x07)<<3);
}

/**
 * Set the watchdog timeout.
 * This function takes a desired input timeout in seconds and returns the closest available timeout from 
 * the margin_set
 */
static inline int WATCHDOG_SET_TIMEOUT(int timeout){
	int div,low,high,med;
	
	for(div=0;div<MARGINNUM ;div++){
		high = (div==MARGINNUM-1)?margin_set[div]:margin_set[div+1];
		low = margin_set[div];
		med = (high+low)/2;
		GENDEBUG("timeout loop div = %x\n high = %u low = %u med = %u\n",div,high,low,med);
		if (timeout<med) {timeout=low;break;}
		//if (timeout<=high) {timeout=high;break;}
	}

	if(timeout>=margin_set[MARGINNUM-1])
		{div=MARGINNUM-1;timeout=margin_set[MARGINNUM-1];}

	GENDEBUG("timeout set to %u\n div = %u\n",timeout,div);
	WDT_SET_PRESCALE_EXP(div);
	return timeout;	
}


#endif 
#endif /*CFCORE_WDT_H_*/
