/****************************************************************************/
/*
 * Flash memory access on uClinux EMAC like devices
 * Copyright (C) 2005, EMAC.Inc <support@emacinc.com>
 * Derived from snapgear-uc.c
 * Copyright (C) 2001-2002, David McCullough <davidm@snapgear.com>
 */
/****************************************************************************/

#include <linux/config.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/reboot.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/nftl.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>
#include <linux/mtd/cfi.h>

#include <linux/fs.h>

#include <linux/major.h>
#include <linux/root_dev.h>

#include <linux/init.h>
#include <asm/io.h>
#include <asm/delay.h>
#include <asm/coldfire.h>  

/****************************************************************************/

#define NB_OF(x)  (sizeof(x)/sizeof(x[0]))

#define SIZE_128K	(1  *  128 * 1024)
#define SIZE_1MB	(1  * 1024 * 1024)
#define SIZE_2MB	(2  * 1024 * 1024)
#define SIZE_4MB	(4  * 1024 * 1024)
#define SIZE_8MB	(8  * 1024 * 1024)
#define SIZE_16MB	(16 * 1024 * 1024)


//#define CFNVRAM_BASE 	0x20000400
//#define SIZE_CFNVRAM		0xf800

#define CFNVRAM_BASE 	0x20001000
#define SIZE_CFNVRAM		0xe000

#define FLASH_BASE_2M	0xffe00000
#define FLASH_BASE_4M	0xffc00000

#define	BUS_WIDTH	2
#define	RAM_BUS_WIDTH	2


/****************************************************************************/


static struct map_info som_flash_map = {
	    .name = "Flash",
};

static struct map_info som_cfram_map = {
	    .name = "CFnvram",
};


static struct mtd_info *flash_mtdinfo;
static struct mtd_info *ram_mtdinfo ;

/****************************************************************************/

#define NUM_PARTITIONS  (sizeof(som_romfs) / sizeof(som_romfs[0]))

/****************************************************************************/
static struct mtd_partition som_cframfs[] = {
{ .name = "CFnvram",   .offset = 0, .size = SIZE_CFNVRAM }
};


/*
 *	The layout of our flash,  note the order of the names,  this
 *	means we use the same major/minor for the same purpose on all
 *	layouts (when possible)
 */

static struct mtd_partition som_2mb[] = {
//	{ .name = "U-Boot", .offset = 0x00000000, .size =   0x00040000 },
//	{ .name = "Kernel", .offset = 0x00040000, .size =    0x180000  },
	{ .name = "FlashFS-256K",   .offset = 0x1c0000, .size =   0x00040000 },
	{ .name = "FlashFS-1M",   .offset = 0x100000, .size =   0x00100000 },
	{ .name = "FlashFS-1.75M",   .offset = 0x040000, .size =   0x001c0000 }
//	{ .name = "Flash",      .offset = 0 }
};

static struct mtd_partition som_4mb[] = {
//	{ .name = "U-Boot", .offset = 0x00000000, .size =   0x00040000 },
//	{ .name = "Kernel", .offset = 0x00040000, .size =    0x200000  },
	{ .name = "FlashFS-256K",   .offset = 0x3c0000, .size =   0x00040000 },
	{ .name = "FlashFS-1M",   .offset = 0x300000, .size =   0x00100000 },
	{ .name = "FlashFS-1.75M",   .offset = 0x240000, .size =   0x001c0000 },
	{ .name = "FlashFS-2M",   .offset = 0x200000, .size =   0x00200000 },
	{ .name = "FlashFS-3.75M",   .offset = 0x040000, .size =   0x002c0000 }
//	{ .name = "Flash",      .offset = 0 }
};

/****************************************************************************/
/*
 * Find the MTD device with the given name
 */

static struct mtd_info *get_mtd_named(char *name)
{
	int i;
	struct mtd_info *mtd;

	for (i = 0; i < MAX_MTD_DEVICES; i++) {
		mtd = get_mtd_device(NULL, i);
		if (mtd) {
			if (strcmp(mtd->name, name) == 0)
				return(mtd);
			put_mtd_device(mtd);
		}
	}
	return(NULL);
}

/****************************************************************************/
#ifdef CONFIG_MTD_CFI_INTELEXT
/*  
 *  Set the Intel flash back to read mode as MTD may leave it in command mode
 */

static int som_reboot_notifier(
    struct notifier_block *nb,
    unsigned long val,
    void *v)
{
    struct cfi_private *cfi = som_flash_map.fldrv_priv;
    int i;
 
    for (i = 0; cfi && i < cfi->numchips; i++)
        cfi_send_gen_cmd(0xff, 0x55, cfi->chips[i].start, &som_flash_map,
                cfi, cfi->device_type, NULL);
    
    return(NOTIFY_OK);
}

static struct notifier_block som_notifier_block = {
    som_reboot_notifier, NULL, 0 
};

#endif

/****************************************************************************/

static int
som_point(struct mtd_info *mtd, loff_t from, size_t len,
		size_t *retlen, u_char **mtdbuf)
{
	struct map_info *map = (struct map_info *) mtd->priv;
	*mtdbuf = (u_char *) (map->map_priv_1 + (int)from);
	*retlen = len;
	return(0);
}

/****************************************************************************/

static int __init
som_probe(int ram, unsigned long addr, int size, int buswidth)

{
	struct mtd_info *mymtd;
	struct map_info *map_ptr;
	
	if (ram)
		map_ptr = &som_cfram_map;
	else	
		map_ptr = &som_flash_map;

	map_ptr->bankwidth = buswidth;
	map_ptr->map_priv_2 = addr;
	map_ptr->phys = addr;
	map_ptr->size = size;

	printk(KERN_NOTICE "EMAC %s probe(0x%lx,%d,%d): %lx at %lx\n",
			ram ? "ram" : "flash",
			addr, size, buswidth, map_ptr->size, map_ptr->map_priv_2);

	map_ptr->virt = (unsigned long)
			ioremap_nocache(map_ptr->map_priv_2, map_ptr->size);

	if (!map_ptr->virt) {
		printk("Failed to ioremap_nocache\n");
		return -EIO;
	}
	

	simple_map_init(map_ptr);

	if(ram)
		mymtd = do_map_probe("map_ram", map_ptr);
	else
		mymtd = do_map_probe("cfi_probe", map_ptr);
	

	if (!mymtd) {
		iounmap((void *)map_ptr->map_priv_1);
		return -ENXIO;
	}
		
	mymtd->owner = THIS_MODULE;
	mymtd->point = som_point;
	mymtd->priv = map_ptr;
	
	if (ram) {
		ram_mtdinfo = mymtd;	
		add_mtd_partitions(mymtd, som_cframfs, NB_OF(som_cframfs));
		return(0);
	}

	flash_mtdinfo = mymtd;	
	
	switch (size) {
	case SIZE_2MB:
		add_mtd_partitions(mymtd, som_2mb, NB_OF(som_2mb));
		break;
	
	case SIZE_4MB:
		add_mtd_partitions(mymtd, som_4mb, NB_OF(som_4mb));
		break;
	}

	return 0;
}

/****************************************************************************/

#define CSAR0                     *(volatile u16 *)(MCF_MBAR + 0x80)

int __init som_mtd_init(void)
{
	int rc = -1;
	struct mtd_info *mtd;
	extern char _ebss;
	
if((CSAR0)==(FLASH_BASE_4M>>16))
	rc = som_probe(0, FLASH_BASE_4M, SIZE_4MB, BUS_WIDTH);

if((CSAR0)==(FLASH_BASE_2M>>16))
	rc = som_probe(0, FLASH_BASE_2M, SIZE_2MB, BUS_WIDTH);
		
if(rc<0){
	printk("EMAC-uc: couldn't locate flash\n");
	printk("unknown address map %x\n",CSAR0);
	}

#ifdef EMAC_MTD_RAM_MAP
if(som_probe(1, CFNVRAM_BASE,SIZE_CFNVRAM,RAM_BUS_WIDTH)<0)
	printk("couldn't map cfnvram\n");
#endif
    
	return(rc);
}

/****************************************************************************/

static void __exit som_mtd_cleanup(void)
{
	if (flash_mtdinfo) {
		del_mtd_partitions(flash_mtdinfo);
		map_destroy(flash_mtdinfo);
		flash_mtdinfo = NULL;
	}

	
	if (som_flash_map.map_priv_1) {
		iounmap((void *)som_flash_map.map_priv_1);
		som_flash_map.map_priv_1 = 0;
	}
}

/****************************************************************************/

module_init(som_mtd_init);
module_exit(som_mtd_cleanup);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("EMAC.Inc <support@emacinc.com>");
MODULE_DESCRIPTION("SoM-5282EM FLASH support for uClinux");

/****************************************************************************/
