/*
 * Platform device registration.
 * Copyright (c) 2006 Michael Broughton (mbobowik@telusplanet.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */


#if defined(CONFIG_M520x)


#include <linux/if_ether.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/kernel.h>
#include <linux/platform_device.h>

#include <asm/coldfire.h>
#include <asm/mcfsim.h>

#include "fec.h"


static unsigned char m520x_fec_mac[] = {
	0x00, 0x14, 0x15, 0x00, 0x01, 0xb5,
};


static unsigned int m520x_fec_phy_id = 1;


static struct platform_device m520x_fec_pdev = {
	.name = "FEC",
	.id	= 0,
	.num_resources = 17,
	.resource = (struct resource[]) {
		FEC_RES_MEM("CSR",
			(unsigned long)(MCF_MBAR + 0x00030000),
			(unsigned long)(MCF_MBAR + 0x000301ff)),
		FEC_RES_MEM("MIB",
			(unsigned long)(MCF_MBAR + 0x00030200),
			(unsigned long)(MCF_MBAR + 0x000303ff)),
#if (CONFIG_MCF_FEC_MAC1 == 0)
		FEC_RES_MEM("MAC",
			(unsigned long)m520x_fec_mac,
			(unsigned long)(m520x_fec_mac + ETH_ALEN - 1)),
#else
		FEC_RES_MEM("MAC",
			(unsigned long)(CONFIG_MCF_FEC_MAC1),
			(unsigned long)(CONFIG_MCF_FEC_MAC1 + ETH_ALEN - 1)),
#endif
		FEC_RES_MEM("PHY_ID",
			(unsigned long)(&m520x_fec_phy_id),
			(unsigned long)(&m520x_fec_phy_id)),
		FEC_RES_IRQ((64 + 36), "fec(TXF)"),
		FEC_RES_IRQ((64 + 37), "fec(TXB)"),
		FEC_RES_IRQ((64 + 38), "fec(UN)"),
		FEC_RES_IRQ((64 + 39), "fec(RL)"),
		FEC_RES_IRQ((64 + 40), "fec(RXF)"),
		FEC_RES_IRQ((64 + 41), "fec(RXB)"),
		FEC_RES_IRQ((64 + 42), "fec(MII)"),
		FEC_RES_IRQ((64 + 43), "fec(LC)"),
		FEC_RES_IRQ((64 + 44), "fec(HBERR)"),
		FEC_RES_IRQ((64 + 45), "fec(GRA)"),
		FEC_RES_IRQ((64 + 46), "fec(EBERR)"),
		FEC_RES_IRQ((64 + 47), "fec(BABT)"),
		FEC_RES_IRQ((64 + 48), "fec(BABR)"),
	},
};


int fec_platform_init(void)
{
	volatile unsigned char  *icrp;
	volatile unsigned long  *imrp;
	int i;

	icrp = (volatile unsigned char *)(MCF_IPSBAR+MCFICM_INTC0+MCFINTC_ICR0);
	for (i = 36; i < 49; i++)
		icrp[i] = 0x04;
	imrp = (volatile unsigned long *)(MCF_IPSBAR+MCFICM_INTC0+MCFINTC_IMRH);
	*imrp &= ~0x0001FFF0;

	*(volatile unsigned char *)(MCF_IPSBAR+MCF_GPIO_PAR_FEC) |= 0xf0;
	*(volatile unsigned char *)(MCF_IPSBAR+MCF_GPIO_PAR_FECI2C) |= 0x0f;

	return platform_device_register(&m520x_fec_pdev);
}


void fec_platform_cleanup(void)
{
	platform_device_unregister(&m520x_fec_pdev);
}


#endif

