/*
 * Platform device registration.
 * Copyright (c) 2006 Michael Broughton (mbobowik@telusplanet.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */


#if defined(CONFIG_M523x)


#include <linux/if_ether.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/kernel.h>
#include <linux/platform_device.h>

#include <asm/coldfire.h>
#include <asm/mcfsim.h>

#include "fec.h"


static unsigned char m523x_fec_mac[] = {
	0x00, 0xcf, 0x52, 0x35, 0xeb, 0x01,
};


static unsigned int m523x_fec_phy_id = 1;


static struct platform_device m523x_fec_pdev = {
	.name = "FEC",
	.id	= 0,
	.num_resources = 17,
	.resource = (struct resource[]) {
		FEC_RES_MEM("CSR",
			(unsigned long)(MCF_MBAR + 0x00001000),
			(unsigned long)(MCF_MBAR + 0x000011ff)),
		FEC_RES_MEM("MIB",
			(unsigned long)(MCF_MBAR + 0x00001200),
			(unsigned long)(MCF_MBAR + 0x000013ff)),
#if (CONFIG_MCF_FEC_MAC1 == 0)
		FEC_RES_MEM("MAC",
			(unsigned long)m523x_fec_mac,
			(unsigned long)(m523x_fec_mac + ETH_ALEN - 1)),
#else
		FEC_RES_MEM("MAC",
			(unsigned long)(CONFIG_MCF_FEC_MAC1),
			(unsigned long)(CONFIG_MCF_FEC_MAC1 + ETH_ALEN - 1)),
#endif
		FEC_RES_MEM("PHY_ID",
			(unsigned long)(&m523x_fec_phy_id),
			(unsigned long)(&m523x_fec_phy_id)),
		FEC_RES_IRQ((64 + 23), "fec(TXF)"),
		FEC_RES_IRQ((64 + 24), "fec(TXB)"),
		FEC_RES_IRQ((64 + 25), "fec(UN)"),
		FEC_RES_IRQ((64 + 26), "fec(RL)"),
		FEC_RES_IRQ((64 + 27), "fec(RXF)"),
		FEC_RES_IRQ((64 + 28), "fec(RXB)"),
		FEC_RES_IRQ((64 + 29), "fec(MII)"),
		FEC_RES_IRQ((64 + 30), "fec(LC)"),
		FEC_RES_IRQ((64 + 31), "fec(HBERR)"),
		FEC_RES_IRQ((64 + 32), "fec(GRA)"),
		FEC_RES_IRQ((64 + 33), "fec(EBERR)"),
		FEC_RES_IRQ((64 + 34), "fec(BABT)"),
		FEC_RES_IRQ((64 + 35), "fec(BABR)"),
	},
};


int fec_platform_init(void)
{
	volatile unsigned char *icrp;
	volatile unsigned long *imrp;
	int i, ilip;

	icrp = (volatile unsigned char *)(MCF_IPSBAR+MCFICM_INTC0+MCFINTC_ICR0);
	for (i = 23, ilip = 0x28; i < 36; i++)
		icrp[i] = ilip--;

	imrp = (volatile unsigned long *)(MCF_IPSBAR+MCFICM_INTC0+MCFINTC_IMRH);
	*imrp &= ~0x0000000f;

	imrp = (volatile unsigned long *)(MCF_IPSBAR+MCFICM_INTC0+MCFINTC_IMRL);
	*imrp &= ~0xff800001;

	return platform_device_register(&m523x_fec_pdev);
}


void fec_platform_cleanup(void)
{
	platform_device_unregister(&m523x_fec_pdev);
}


#endif

