/*
 * Platform device registration.
 * Copyright (c) 2006 Michael Broughton (mbobowik@telusplanet.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */


#if defined(CONFIG_M532x)


#include <linux/if_ether.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/kernel.h>
#include <linux/platform_device.h>

#include <asm/coldfire.h>
#include <asm/mcfsim.h>

#include "fec.h"


static unsigned char m532x_fec_mac[] = {
	0x00, 0x14, 0x15, 0x00, 0x01, 0xb5,
};


static unsigned int m532x_fec_phy_id = 1;


static struct platform_device m532x_fec_pdev = {
	.name = "FEC",
	.id	= 0,
	.num_resources = 17,
	.resource = (struct resource[]) {
		FEC_RES_MEM("CSR",
			(unsigned long)(MCF_MBAR + 0x00030000),
			(unsigned long)(MCF_MBAR + 0x000301ff)),
		FEC_RES_MEM("MIB",
			(unsigned long)(MCF_MBAR + 0x00030200),
			(unsigned long)(MCF_MBAR + 0x000303ff)),
#if (CONFIG_MCF_FEC_MAC1 == 0)
		FEC_RES_MEM("MAC",
			(unsigned long)m532x_fec_mac,
			(unsigned long)(m532x_fec_mac + ETH_ALEN - 1)),
#else
		FEC_RES_MEM("MAC",
			(unsigned long)(CONFIG_MCF_FEC_MAC1),
			(unsigned long)(CONFIG_MCF_FEC_MAC1 + ETH_ALEN - 1)),
#endif
		FEC_RES_MEM("PHY_ID",
			(unsigned long)(&m532x_fec_phy_id),
			(unsigned long)(&m532x_fec_phy_id)),
		FEC_RES_IRQ((64 + 36), "fec(TXF)"),
		FEC_RES_IRQ((64 + 37), "fec(TXB)"),
		FEC_RES_IRQ((64 + 38), "fec(UN)"),
		FEC_RES_IRQ((64 + 39), "fec(RL)"),
		FEC_RES_IRQ((64 + 40), "fec(RXF)"),
		FEC_RES_IRQ((64 + 41), "fec(RXB)"),
		FEC_RES_IRQ((64 + 42), "fec(MII)"),
		FEC_RES_IRQ((64 + 43), "fec(LC)"),
		FEC_RES_IRQ((64 + 44), "fec(HBERR)"),
		FEC_RES_IRQ((64 + 45), "fec(GRA)"),
		FEC_RES_IRQ((64 + 46), "fec(EBERR)"),
		FEC_RES_IRQ((64 + 47), "fec(BABT)"),
		FEC_RES_IRQ((64 + 48), "fec(BABR)"),
	},
};


int fec_platform_init(void)
{
	/* Unmask interrupts */
	MCF_INTC0_ICR36 = 0x2;
	MCF_INTC0_ICR37 = 0x2;
	MCF_INTC0_ICR38 = 0x2;
	MCF_INTC0_ICR39 = 0x2;
	MCF_INTC0_ICR40 = 0x2;
	MCF_INTC0_ICR41 = 0x2;
	MCF_INTC0_ICR42 = 0x2;
	MCF_INTC0_ICR43 = 0x2;
	MCF_INTC0_ICR44 = 0x2;
	MCF_INTC0_ICR45 = 0x2;
	MCF_INTC0_ICR46 = 0x2;
	MCF_INTC0_ICR47 = 0x2;
	MCF_INTC0_ICR48 = 0x2;

	MCF_INTC0_IMRH &= ~(
		MCF_INTC_IMRH_INT_MASK36 |
		MCF_INTC_IMRH_INT_MASK37 |
		MCF_INTC_IMRH_INT_MASK38 |
		MCF_INTC_IMRH_INT_MASK39 |
		MCF_INTC_IMRH_INT_MASK40 |
		MCF_INTC_IMRH_INT_MASK41 |
		MCF_INTC_IMRH_INT_MASK42 |
		MCF_INTC_IMRH_INT_MASK43 |
		MCF_INTC_IMRH_INT_MASK44 |
		MCF_INTC_IMRH_INT_MASK45 |
		MCF_INTC_IMRH_INT_MASK46 |
		MCF_INTC_IMRH_INT_MASK47 |
		MCF_INTC_IMRH_INT_MASK48 );

	/* Set up gpio outputs for MII lines */
	MCF_GPIO_PAR_FECI2C |= (0 |
		MCF_GPIO_PAR_FECI2C_PAR_MDC_EMDC |
		MCF_GPIO_PAR_FECI2C_PAR_MDIO_EMDIO);
	MCF_GPIO_PAR_FEC = (0 |
		MCF_GPIO_PAR_FEC_PAR_FEC_7W_FEC |
		MCF_GPIO_PAR_FEC_PAR_FEC_MII_FEC);

	return platform_device_register(&m532x_fec_pdev);
}


void fec_platform_cleanup(void)
{
	platform_device_unregister(&m532x_fec_pdev);
}


#endif

