/***************************************************************************
                          comedi_1290.c  -  
Driver: ltc1290.ko
Description: Analog to Digital driver for the LTC1290
Devices:LTC1290 AtoD SPI device
Author: EMAC.Inc->NZG
Status: untested

This driver is a generic plug in for the LTC1290 SPI based AtoD.
Low level hardware functions are implemented in ltc1290.c 

Configuration Options:	none

    					  (C) Copyright 2005, EMAC.Inc www.emacinc.com   
                             -------------------
    begin                : Mon August 1 2005
    email                : ngustavson@emacinc.com

 ***************************************************************************/

/***************************************************************************
 *                                                                         
 *   This program is free software; you can redistribute it and/or modify  
 *   it under the terms of the GNU General Public License as published by  
 *   the Free Software Foundation; either version 2 of the License, or     
 *   (at your option) any later version.                                   
 *                                                                         
 ***************************************************************************/
 
#include <linux/comedidev.h>
#include "ltc1290.h"

/* Imaginary registers for the imaginary board */

#define SKEL_SIZE 0

#define SKEL_START_AI_CONV	0
#define SKEL_AI_READ		0

typedef struct ltc1290_board_struct{
	char *name;
	int ai_chans;
	int ai_bits;
	int have_dio;
	ltc_dev *ltc;
}ltc1290_board;

static ltc1290_board ltc1290_boards[] = {
	{
	name:		"SoM-100ESR2",
	ai_chans:	8,
	ai_bits:	12,
	have_dio:	0,
	},
};



/*
 * Useful for shorthand access to the particular board structure
 */
#define thisboard(dev) ((ltc1290_board *)dev->board_ptr)

/* this structure is for data unique to this hardware driver.  If
   several hardware drivers keep similar information in this structure,
   feel free to suggest moving the variable to the comedi_device struct.  */
typedef struct{
	int data;
}ltc1290_private;
/*
 * most drivers define the following macro to make it easy to
 * access the private structure.
 */
#define devpriv(dev) ((ltc_1290_private *)dev->private)

/*
 * The comedi_driver structure tells the Comedi core module
 * which functions to call to configure/deconfigure (attach/detach)
 * the board, and also about the kernel module that contains
 * the device code.
 */
static int ltc1290_attach(comedi_device *dev,comedi_devconfig *it);
static int ltc1290_detach(comedi_device *dev);
static comedi_driver driver_ltc1290={
	driver_name:	"ltc1290",
	module:			THIS_MODULE,
	attach:				ltc1290_attach,
	detach:			ltc1290_detach,
	board_name:	ltc1290_boards,
	offset:				sizeof(ltc1290_board),
	num_names:	sizeof(ltc1290_boards) / sizeof(ltc1290_board),
};

/*
 * "instructions" read/write data in "one-shot" or "software-triggered"
 * mode.
 */
static int ltc1290_ai_rinsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *insn,lsampl_t *data)
{
	int channel = CR_CHAN(insn->chanspec);
	ltc_dev *ltc = thisboard(dev)->ltc;
	int sample;	
	for(sample=0;sample<insn->n;sample++)
		data[sample] = LTC1290_Read_Chan(ltc,channel);

	/* return the number of samples read/written */
	return sample;
}

/*
 * Attach is called by the Comedi core to configure the driver
 * for a particular board. 
 */
static int ltc1290_attach(comedi_device *dev,comedi_devconfig *it)
{
	comedi_subdevice *s;

	printk("comedi%d: skel: ",dev->minor);
	
/*
 * Initialize dev->board_name.  Note that we can use the "thisboard"
 * macro now, since we just initialized it in the last line.
 */
	dev->board_name = thisboard(dev)->name;

/*
 * Allocate the private structure area.  alloc_private() is a
 * convenient macro defined in comedidev.h.
 */
	if(alloc_private(dev,sizeof(ltc1290_private))<0)
		return -ENOMEM;

/*
 * Allocate the subdevice structures.  alloc_subdevice() is a
 * convenient macro defined in comedidev.h.
 */
	if(alloc_subdevices(dev, 3)<0)
		return -ENOMEM;

	s=dev->subdevices+0;
	/* analog input subdevice */
	s->type=COMEDI_SUBD_AI;
	/* we support single-ended (ground) and differential */
	s->subdev_flags=SDF_READABLE|SDF_GROUND|SDF_DIFF;
	s->n_chan=thisboard(dev)->ai_chans;
	s->maxdata=(1<<thisboard(dev)->ai_bits)-1;
	s->range_table=&range_bipolar10;
	s->len_chanlist=16;  /* This is the maximum chanlist length that
				the board can handle */
	s->insn_read = ltc1290_ai_rinsn;

	s=dev->subdevices+1;
	/* analog output subdevice */
	s->type = COMEDI_SUBD_UNUSED;

	s=dev->subdevices+2;
	s->type = COMEDI_SUBD_UNUSED;
	
	 thisboard(dev)->ltc =LTC1290_board_init();
	
	printk("attached\n");

	return 1;
}


/*
 * detach is called to deconfigure a device.  It should deallocate
 * resources.  
 * This function is also called when _attach() fails, so it should be
 * careful not to release resources that were not necessarily
 * allocated by _attach().  dev->private and dev->subdevices are
 * deallocated automatically by the core.
 */
static int ltc1290_detach(comedi_device *dev)
{
	printk("comedi%d: ltc1290: remove\n",dev->minor);	
	return 0; 
}


/*
 * A convenient macro that defines init_module() and cleanup_module(),
 * as necessary.
 */
COMEDI_INITCLEANUP(driver_ltc1290);
 