/***************************************************************************
                      									DS1722.c
              DS1722 Temperature sensor driver for uClinux mcfqspi control
   ---------------------------
    begin                				: Fri May 5 2006
    email                				: support@emacinc.com
    (C) Copyright 2006		: EMAC.Inc - www.emacinc.com   
   ---------------------------
 ***************************************************************************/
#include <linux/slab.h>
#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/slab.h>
#include <linux/version.h>
#include <linux/proc_fs.h>
#include <asm/uaccess.h> 
#include <linux/miscdevice.h>
#include "ds1722.h"

static int qcs __initdata = 0x300;	

module_param(qcs, int, 0);
MODULE_PARM_DESC(qcs, "chip select number passed to the qspi");

/*!Tranceive DS1722 register
@param device qspi device the DS1722 belongs too
@param reg he register to write to(from the enumerated registers above)
@return the byte that was read
*/
static u8 ds1722_tranceive_register(ds1722_qspi_device *dev,u8 address,u8 data )
{
u8 buffer[2];
buffer[0] = address;
buffer[1] = data;
dev->qspi->read_data.length = sizeof(buffer); 
dev->qspi->read_data.buf = buffer;

qspi_mutex_down();
CSDIP(dev->cs);
qspi_internal_read(dev->qspi,buffer, sizeof(buffer),0,dev->cs);
qspi_mutex_up();

dev->qspi->read_data.length=0;
dev->qspi->read_data.buf =NULL;

return buffer[1];
}

static ds1722_qspi_device *ds1722_create_device(int cs)
{
ds1722_qspi_device *dev=NULL;	
qspi_dev *spi =  qspi_create_device();

if(spi==NULL)
	return(NULL); 
        
if ((dev = kmalloc(sizeof(ds1722_qspi_device), GFP_KERNEL)) == NULL) {  
	qspi_destroy_device(spi);
	return(NULL);
}
       		
dev->qspi = spi;
dev->cs = cs;

spi->poll_mod = 1;
spi->baud = qspi_BAUD(4000000);    /* baud rate 4M */
spi->cpha      =  1;    /* SPI clock phase */
                
ds1722_tranceive_register(dev,CONTROLW,(EIGHTBIT|CONTINOUS_CONVERSION));
return dev;
}


static ds1722_qspi_device  *ds1722_destroy_device(ds1722_qspi_device *dev){
if(dev==NULL)
	return NULL;
qspi_destroy_device(dev->qspi);
kfree(dev);	
return NULL;
}
/*!Get the current temperature from the ds1722
Temperture is returned in degrees C. The device is capable of returning a higher resolution but is 
only accurate +-2 degrees.
@return temperature in Celcius
*/
static int ds1722_gettemp(ds1722_qspi_device *dev){
	return ds1722_tranceive_register(dev,DATAH,0);
}

/* Handle read from proc file */
static int readProcds1722C( char *buffer, char **start, off_t offset, int size, int *eof, void *data ){
	ds1722_qspi_device *dev = data;
	return sprintf( buffer, "%u", ds1722_gettemp(dev));
}

static int readProcds1722F( char *buffer, char **start, off_t offset, int size, int *eof, void *data ){
	ds1722_qspi_device *dev = data;
	return sprintf( buffer, "%u", (ds1722_gettemp(dev)*9)/5 +32);
}

ds1722_qspi_device *ds1722dev; 
struct proc_dir_entry *procbase;

static int __init ds1722_init(void){
procbase  = proc_mkdir( "ds1722",NULL);
ds1722dev  =ds1722_create_device(qcs);

if(ds1722dev==NULL){
	printk("couldn't create ds1722 device\n");
	return -1;
}

//printk("device cs = %x\n",qcs);
//printk("device qcs = %x\n",ds1722dev->cs);

printk(" "DSNAME " driver version " DS_DRIVER_V " (c) " __DATE__  "\n");
	printk("EMAC.inc (support@emacinc.com)\n");

		if( !create_proc_read_entry( "Farenheit", 0, procbase, readProcds1722F,(void *)ds1722dev ) ){
			printk( KERN_ERR "ds1722: Error! Can't create /proc/ds1722/Farenheit\n" );
			return -ENOMEM;
		}

		if( !create_proc_read_entry( "Celcius", 0, procbase, readProcds1722C,(void *)ds1722dev ) ){
			printk( KERN_ERR "ds1722: Error! Can't create /proc/ds1722/Celcius\n" );
			return -ENOMEM;
		}

return 0;
}

static void __exit ds1722_exit (void){
remove_proc_entry( "Celcius", procbase );
remove_proc_entry( "Farenheit", procbase );
remove_proc_entry( "ds1722", NULL );
ds1722_destroy_device(ds1722dev);
printk(DSNAME" driver unloaded\n");
}

module_init(ds1722_init);
module_exit(ds1722_exit);


