/***************************************************************************
                          ltc1290.c  -  SPI AtoD device function calls
                          Depends on the stackable qspi driver
    					  (C) Copyright 2005, EMAC.Inc www.emacinc.com   
                          Modified up from the pcm-37e12 version
                             -------------------
    begin                : Thur July 28 2005
    email                : ngustavson@emacinc.com

 ***************************************************************************/

/***************************************************************************
 *                                                                         
 *   This program is free software; you can redistribute it and/or modify  
 *   it under the terms of the GNU General Public License as published by  
 *   the Free Software Foundation; either version 2 of the License, or     
 *   (at your option) any later version.                                   
 *                                                                         
 ***************************************************************************/
#include <linux/slab.h>
#include <asm/ltc1290.h>

const char ltc_channels[NUMCHANNELS] = {0x80,0xC0,0x90,0xD0,0xA0,0xE0,0xB0,0xF0};

const u16 command[NUMCHANNELS+1] = {
	(0x80|READCMD)<<8,
	(0xC0|READCMD)<<8,
	(0x90|READCMD)<<8,
	(0xD0|READCMD)<<8,
	(0xA0|READCMD)<<8,
	(0xE0|READCMD)<<8,
	(0xB0|READCMD)<<8,
	(0xF0|READCMD)<<8,
	(0x80|READCMD)<<8
	};

ltc_dev *LTC_SPIDevice(int qcr_cs){
	qspi_dev *spi;
	ltc_dev *ltc;
	
	if((spi=qspi_create_device())==NULL){
		printk("couldn't create qspi device");
		return(NULL);
	}
	
	if ((ltc = kmalloc(sizeof(ltc_dev), GFP_KERNEL)) == NULL) {//create ltc device  
                qspi_destroy_device(spi);
                return(NULL);
        }
	
	ltc->qcr_cs = qcr_cs<<8;
	
	spi->bits = 16;    /* 16 bits transfers to transfer for each entry */
	spi->baud = qspi_BAUD(2000000);    /* setup bit rate - 2M */
	spi->qcd = 127;                     /* no QSPILCK delay */
	spi->dtl = 30;                     /* no delay after transfer */
	spi->qcr_cont = 1;      /* keep CS active throughout transfer */
	spi->odd_mod  = 0;    /* if length of buffer is a odd number, 16-bit transfers are finalized with a 8-bit transfer */
	spi->dsp_mod  = 0;    /* transfers are bounded to 15/30 bytes
                                           (= a multiple of 3 bytes = 1 word) */
    //spi->poll_mod  =  0;    /*interrupt mode */
     spi->poll_mod  =  1;    /*polled mode */
     spi->cpol      =  1;    /* SPI clock polarity */
     spi->cpha      =  0;    /* SPI clock phase */
     spi->dohie     =  0;    /* data output high impedance enable */
      
      ltc->spi = spi;
         
return ltc;
}


ltc_dev *LTC_Destroy_Device(ltc_dev *dev){
	kfree(dev->spi);
	kfree(dev);
	return NULL;
}

u16 *LTC1290_Read_All(ltc_dev *device){
device->spi->read_data.buf = (u8 *)command;
device->spi->read_data.loop	= 0;	
device->spi->read_data.length = sizeof(command);	
qspi_mutex_down();
qspi_internal_read(device->spi,(u8 *)device->buffer,sizeof(device->buffer),
			NULL,device->qcr_cs);	
qspi_mutex_up();	
return &device->buffer[1];
}

u16 LTC1290_Read_Chan(ltc_dev *device,int channel){
device->spi->read_data.buf = (u8 *)&command[channel];
device->spi->read_data.loop	= 0;	
device->spi->read_data.length = 2*sizeof(command[channel]);	
qspi_mutex_down();
qspi_internal_read(device->spi,(u8 *)&device->buffer[channel-1],sizeof(2*sizeof(command[channel])),
			NULL,device->qcr_cs);	
qspi_mutex_up();	
return device->buffer[channel];
}



