/***************************************************************************
                          ltc1290_char.c  -  character driver interface to the ltc1290 
    					  (C) Copyright 2005, EMAC.Inc www.emacinc.com   
                          Modified up from the pcm-37e12 version
                             -------------------
    begin                : Thur July 28 2005
    email                : ngustavson@emacinc.com

 ***************************************************************************/

/***************************************************************************
 *                                                                         
 *   This program is free software; you can redistribute it and/or modify  
 *   it under the terms of the GNU General Public License as published by  
 *   the Free Software Foundation; either version 2 of the License, or     
 *   (at your option) any later version.                                   
 *                                                                         
 ***************************************************************************/
 #include <linux/module.h>/* include file for kernel development*/
#include <linux/kernel.h>/* printk and others*/
#include <linux/fs.h>    /* the file structure */
#include <linux/ioport.h> /* inb, outb */
#include <linux/delay.h>
#include <linux/interrupt.h>
//#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/cdev.h>
#include <asm/io.h> /*ioremap*/
#include <asm/uaccess.h>

#define CHARACTER_INTERFACE
#include <asm/ltc1290.h>

#define DRIVER_VERSION "1.0"
#define DRIVER_NAME "char1290"

MODULE_AUTHOR("EMAC.Inc-NZG");
MODULE_LICENSE("Dual BSD/GPL");

#undef LTCCDEBUG
#ifdef LTCC_DEBUG
#define LTCCDEBUG(fmt, args...) printk(fmt"\n", ## args)
#else 
#define LTCCDEBUG(fmt, args...) 
#endif

/**
 * First Minor number for GPIO devices
 */
 #define DAQMAJOR 101
 #define FIRSTLTCMINOR 0
 #define LTCMINORNUM 1

static struct cdev *ltc1290cdev;


/**
 * LTC1290_IOCTL
 * LTC1290 control and data exchange with user space via standard ioctl call
 * additional interfaces could be added here fairly easily.
 */
static int LTC1290_IOCTL(struct inode *inode, struct file *filp, unsigned int cmd,
                unsigned long arg){

ltc_dev *dev = filp->private_data;

LTCCDEBUG("LTC1290_IOCTL\n");

switch(cmd){
	case READALL:{
	u16 *buffer = LTC1290_Read_All(dev);
	int length = NUMCHANNELS*(sizeof(u16));
	if(access_ok(VERIFY_WRITE,(void *)arg,length)){
		LTCCDEBUG("copy to user %u bytes\n",length);
		copy_to_user((u16 *)arg,buffer,length);/*move data to user space*/
		return 0;
	}else{
		printk("couldn't write to user buffer\n");
		return -EINVAL;
	}}
	
	case GETCHANNELNUM:{
		int channum = NUMCHANNELS;
		LTCCDEBUG("GETCHANNELNUM\n");
		if(access_ok(VERIFY_WRITE,(void *)arg,(sizeof(int)))){
				LTCCDEBUG("access ok\n");
				put_user(channum,(int *)arg);
				return 0;
	}else{
		printk("couldn't write to user buffer\n");
		return -EINVAL;
	}}
	
}
return -EINVAL;
}

static int LTC1290_Open (struct inode *inode, struct file *filp){
	ltc_dev *dev =LTC1290_board_init();
	  
	  LTCCDEBUG("LTC1290_Open\n");
	  
	  if(dev==NULL){
          		printk("couldn't create ltc device\n");
                return(-ENOMEM);
        }       
	filp->private_data = dev;        
return 0;//don't need anything special here
}

static int LTC1290_Close(struct inode *inode, struct file *filp){
	ltc_dev *dev = filp->private_data;
	filp->private_data  = LTC_Destroy_Device(dev);
return 0;//don't need anything special here
}


struct file_operations LTC1290_Fops={
  .owner =    THIS_MODULE,
  .open   =    LTC1290_Open,
  .ioctl   =      LTC1290_IOCTL,
  .release =  LTC1290_Close,
};


static int LTC1290_init_module(void){
dev_t dev = MKDEV(DAQMAJOR ,FIRSTLTCMINOR);
int result;

ltc1290cdev = cdev_alloc();

printk(DRIVER_NAME" driver - version "DRIVER_VERSION" (c) " __DATE__  "\n");
printk(" N.Z. Gustavson (ngustavson@emacinc.com), EMAC.Inc\n");

result = register_chrdev_region(dev, LTCMINORNUM,DRIVER_NAME);

if(result < 0){
	printk(DRIVER_NAME" driver failed to register Major Number"); 
	return result; 
	}

ltc1290cdev->ops = &LTC1290_Fops;
ltc1290cdev->owner = THIS_MODULE;

cdev_add(ltc1290cdev,dev,LTCMINORNUM);
return 0;
}

/*cleanup_module
this routine is called when a module is unloaded with rmmod
*/
static void LTC1290_cleanup_module(void){
	dev_t devno = MKDEV(DAQMAJOR ,FIRSTLTCMINOR);
	unregister_chrdev_region(devno, LTCMINORNUM);
	cdev_del(ltc1290cdev);
}

module_init(LTC1290_init_module);
module_exit(LTC1290_cleanup_module);
