/************************************************************************
*                                                                     		 
*  mcf_mmc.c - MMC QSPI driver  						
*  MCF5272, MCF5235, MCF5282   									
*                                                                      
*  (C) Copyright 2005, EMAC.Inc www.emacinc.com                       
*  written by		NZG ngustavson@emacinc.com		                
************************************************************************/

#include <linux/config.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>

#include <linux/kernel.h>	
#include <linux/slab.h>		
#include <linux/fs.h>		
#include <linux/errno.h>	
#include <linux/types.h>	
#include <linux/fcntl.h>	
#include <linux/hdreg.h>	
#include <linux/genhd.h>
#include <linux/blkdev.h>
#include <linux/buffer_head.h>	
#include <linux/bio.h>
#include <linux/workqueue.h>
#include <linux/moduleparam.h>


#include "mcf_mmc_prot.h"

//#define TDELAY(a,b) modtimer(a,b)
//#define TDELAY(a,b) 

/***************************************************************************
 * External Functions from mcf_mmc_prot
 ***************************************************************************/
extern mmc_dev *MMC_Get_Device(int channel);

extern mmc_dev *MMC_Destroy_Device(mmc_dev *dev);

extern int MMC_BlockRead(mmc_dev *device, int command, int arg1, int arg2,
		int response, u8 *data,int blocksize, int blocknum,int Nac);	
		
extern int MMC_BlockWrite(mmc_dev *device,int command, int arg1, int arg2, 
		int response, u8 *data, int blocksize, int blocknum,int Nwr);			


#define DEVICE_NAME "mcf_mmc"
#define DEVICE_NR(device) (MINOR(device))
#define DEVICE_ON(device)
#define DEVICE_OFF(device)
#define MAJOR_NR 121

/*
 * We can tweak our hardware sector size, but the kernel talks to us
 * in terms of small sectors, always.
 */
#define KERNEL_SECTOR_SIZE	512

MODULE_AUTHOR("N.Z. Gustavson <ngustavsonr@emacinc.com");
MODULE_DESCRIPTION("Driver for MMC/SD-Cards");
MODULE_SUPPORTED_DEVICE("Motorola MCF5282");
MODULE_SUPPORTED_DEVICE("Motorola MCF5272");
MODULE_LICENSE("Dual BSD/GPL");

//#define MMCB_DEBUG 
 
#undef MMCBDEBUG
#ifdef MMCB_DEBUG
#define MMCBDEBUG(fmt, args...) printk(fmt"\n", ## args)
#else 
#define MMCBDEBUG(fmt, args...) 
#endif

#define MMC_MINORS 16
#define MCFMMC_MAXDEVICES 4

int mmc_devices[MCFMMC_MAXDEVICES]={2,0,0,0};//default to only probe CS1 (SoM-100ES location)
__MODULE_PARM_TYPE(mmc_devices, "1-4i");

static mmc_dev *global_device_ptr[MCFMMC_MAXDEVICES];

//static struct timer_list mmc_timer;

/**
 * Prototypes - may want to make these static functions
 */
void mmc_request(request_queue_t *q);
int mmc_open(struct inode *inode, struct file *filp);
int mmc_release(struct inode *inode, struct file *filp);
int mmc_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
 		unsigned long arg);
int mmc_check_media_change(struct gendisk *gd);
int mmc_revalidate(struct gendisk *gd);
int __init mmc_driver_init(void);
void __exit mmc_driver_exit(void);


static int mcf_mmc_transfer(struct mmc_dev *dev, unsigned long sector,
		unsigned long nsect, char *buffer, int write){
			
	unsigned long offset = sector*KERNEL_SECTOR_SIZE;
	unsigned long nbytes = nsect*KERNEL_SECTOR_SIZE;		
	unsigned long blocks = nbytes/dev->blocklength;
	
	MMCBDEBUG("mmc transfer\n");
	MMCBDEBUG("mmc sector %lu\n",sector);
	MMCBDEBUG("mmc nsect %lu\n",nsect);
	MMCBDEBUG("mmc nbytes %lu\n",nbytes);
    MMCBDEBUG("dev->blocklength %u\n",dev->blocklength);
	
	
			
	if ((offset + nbytes) > dev->size) {
		printk ("Beyond-end write (%ld %ld)\n", offset, nbytes);
		return -1;
	}
	
	if (blocks==0) {
		printk ("Device blocklength>%u not supported\n",KERNEL_SECTOR_SIZE);
		return -1;
	}
		
	if (write)
		return(WRITE_MULTIPLE_BLOCK(dev,offset, buffer, dev->blocklength, blocks));
	else
		return(READ_MULTIPLE_BLOCK(dev,offset,buffer,dev->blocklength,blocks));	
}


void mmc_request(request_queue_t *q){
		
	struct request *req;		
	MMCBDEBUG("mmc_request\n"); 
	
		while ((req = elv_next_request(q)) != NULL) {
			struct mmc_dev *dev = req->rq_disk->private_data;				
				
		if ((dev->state&CHANGED) ||(!(dev->state&VALID))){
			printk("Device is not currently valid\n");
			end_request(req, 0);
			continue;
		}
		
		if (!blk_fs_request(req)) {
			MMCBDEBUG (KERN_NOTICE "Skip non-fs request\n");
			end_request(req, 0);
			continue;
		}
					
		
			if((mcf_mmc_transfer(dev, req->sector, req->current_nr_sectors,
					req->buffer, rq_data_dir(req)))<0){							
				end_request(req,0);//failure				
				}
			else
				end_request(req,1);	//success				
		}
	}


int mmc_open(struct inode *inode, struct file *filp)
{
	struct mmc_dev *dev = inode->i_bdev->bd_disk->private_data;
	struct gendisk
	 *gd=dev->private_data;	
	
	MMCBDEBUG("mmc_open\n"); 
	
	filp->private_data = dev;
//	del_timer_sync(&mmc_timer);
	spin_lock(&dev->lock);
	 
	check_disk_change(inode->i_bdev);
	
	if ((dev->state&CHANGED) ||(!(dev->state&VALID))){
			printk ("Device is not currently valid, failing open\n");
			spin_unlock(&dev->lock);
			//mmc_timer.expires = jiffies + HZ;
			//add_timer(&mmc_timer);
			//TDELAY(&mmc_timer,jiffies + HZ);
			return -ENODEV;
		}
	
	set_capacity(gd, dev->sectors*(dev->blocklength/KERNEL_SECTOR_SIZE));
	MMCBDEBUG("set capacity to %u\n",dev->sectors*(dev->blocklength/KERNEL_SECTOR_SIZE));
	MMCBDEBUG("sectors=%u\n",dev->sectors);
	MMCBDEBUG("CSIZE=%u\n",dev->CSD.C_SIZE);
	MMCBDEBUG("CMULT=%u\n",dev->CSD.C_SIZE_MULT);
	
	dev->users++;
	spin_unlock(&dev->lock);
	//mmc_timer.expires = jiffies + HZ;
	//add_timer(&mmc_timer);
	//TDELAY(&mmc_timer,jiffies + HZ);
		
	return 0;
}

int mmc_release(struct inode *inode, struct file *filp)
{
	struct mmc_dev *dev = inode->i_bdev->bd_disk->private_data;
	//mmc_dev *dev = filp->private_data;
	
	MMCBDEBUG("mmc_release - getting lock\n"); 
	spin_lock(&dev->lock);//locks driver for some reason
	MMCBDEBUG("obtained lock\n"); 
	dev->users--;
	spin_unlock(&dev->lock);
	MMCBDEBUG("released lock\n"); 
	
	return 0;
}

int mmc_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg)
{
	struct hd_geometry geometry;
	mmc_dev *dev = filp->private_data;
	struct block_device *bdev = inode->i_bdev;
	
	MMCBDEBUG("mmc_ioctl\n"); 
	
	if ((dev->state&CHANGED) ||(!(dev->state&VALID))){
			printk ("Device is not currently valid\n");
			return -ENODEV;
		}
	

	if (cmd == BLKGETSIZE)
	{
		MMCBDEBUG("   BLKGETSIZE\n");
		if (!arg) return -EINVAL;
		if (copy_to_user((long *) arg, (long *) &dev->blocklength, sizeof(dev->blocklength))) return -EFAULT;
		return 0;
	}
	else if (cmd == HDIO_GETGEO)
	{
		MMCBDEBUG("   HDIO_GETGEO\n");
		if (!arg) return -EINVAL;
		
		geometry.cylinders	= get_capacity(bdev->bd_disk) / (4 * 16);
		geometry.heads	= 4;
		geometry.sectors	= 16;
		geometry.start	= get_start_sect(bdev);
		
		if (copy_to_user((void *) arg, &geometry, sizeof(geometry))) return -EFAULT;
		return 0;
	}
	else return -ENOTTY; /* unknown command */
}


void mmc_check_media(void *data);
DECLARE_WORK(media_check,mmc_check_media,NULL);

/******************************************************************************
 * Removable media support- requires SoM CPLD pof Rev2 to work correctly
 * This CPLD version is shipped on all SoM-100ES's made after June 16 2005
 * If an older version is used, the device will not correctly detect removal, and the
 * driver will have to be reloaded to correctly re-initialize the device
 ******************************************************************************/

void mmc_check_media(void *data){
    int device;
MMCBDEBUG("checking for media change\n");
   for(device=0;device<MCFMMC_MAXDEVICES;device++){
  		int state;
  		mmc_dev *dev;
  		
  		MMCBDEBUG("device %x\n",device);
  		
  		if(global_device_ptr==NULL){
  			MMCBDEBUG("MMC ERROR:no devices available\n");
  			continue;	
  		}
  		
  		if(!(dev=global_device_ptr[device]))
  			continue;
  		
  		MMCBDEBUG("retrieving state\n");
  				
  		state = MMC_Detect(dev->qcr_cs);//original	
  		
  		if(state==(dev->state&VALID))
  			continue;//same state, no change
  
  		printk("Multimedia card %s\n",state==VALID ? "inserted":"removed");	
  	
  		dev->state = state|CHANGED;				  			
}
	//del_timer(&mmc_timer);
	//mmc_timer.expires = jiffies + HZ;
	//add_timer(&mmc_timer);
	//TDELAY(&mmc_timer,jiffies + HZ);
	MMCBDEBUG("reschedule\n");
	schedule_delayed_work(&media_check,HZ);
}




int mmc_check_media_change(struct gendisk *gd){
struct mmc_dev *dev = gd->private_data;
return(dev->state&CHANGED ? 1:0);		
}

int mmc_revalidate(struct gendisk *gd){
struct mmc_dev *dev = gd->private_data;
if (dev->state==(CHANGED|VALID)){
	printk("MMC media change detected, reinitializing\n");
	return(MMC_Init(dev,dev->qcr_cs>>8));	
}
return -ENODEV;
}

/******************************************************************************/


static struct block_device_operations mmc_bdops = {
	.owner           		= THIS_MODULE,
	.open						=  mmc_open,
	.release					=  mmc_release,
	.ioctl 						=  mmc_ioctl,
	.media_changed	= mmc_check_media_change,
	.revalidate_disk	= mmc_revalidate,
};



static mmc_dev *setup_device(int which){
	mmc_dev *dev;
	struct gendisk *gd=NULL;		
	
	MMCBDEBUG("setup device %u\n",which);
	
	if((dev=MMC_Get_Device(which))==NULL){
		printk("failed to create MMC device %d\n", which);
		return NULL;
	}
			 
	dev->private_data = gd = alloc_disk(MMC_MINORS);
	gd->private_data = dev;
	gd->major = MAJOR_NR;
	gd->first_minor = which*MMC_MINORS;
	gd->fops = &mmc_bdops;
	gd->flags = GENHD_FL_REMOVABLE;
	
	gd->queue = blk_init_queue(mmc_request,&dev->lock);		
	blk_queue_hardsect_size(gd->queue,dev->blocklength);
	gd->queue->queuedata = dev;
	
	snprintf (gd->disk_name, 32, "mcf_mmc%c", which + 'a');
	
	if(dev->state==VALID)
		set_capacity(gd, dev->sectors*(dev->blocklength/KERNEL_SECTOR_SIZE));

	add_disk(gd);
	
	return dev;
}




static void remove_device(mmc_dev *dev){
	struct gendisk *gd=dev->private_data;	
	struct request_queue *queue = gd->queue;
	
	if (gd) {//remove the gendisk
		del_gendisk(gd);
		put_disk(gd);
		}			
			
	if (queue) //remove it's queue
		blk_cleanup_queue(queue);

	MMC_Destroy_Device(dev);//destroy the MMC device
}


int __init mmc_driver_init(void)
{
	int rc;
	int device;
	int devices=0;

	MMCBDEBUG("mmc_driver_init\n");
	rc = register_blkdev(MAJOR_NR, DEVICE_NAME);
	if (rc < 0)
	{
		printk(KERN_WARNING "mcf_mmc: can't get major %d\n", MAJOR_NR);
		return rc;
	}

	//init_timer(&mmc_timer);
	//mmc_timer.function = (void *)mmc_check_media;
	//mmc_timer.function = mmc_check_media;
	
	for(device=0;device<MCFMMC_MAXDEVICES;device++)
		if(mmc_devices[device]){			
			global_device_ptr[device] = setup_device(mmc_devices[device]);
			if(global_device_ptr[device])
				devices++;//number of devices created
		}else global_device_ptr[device] = NULL;
		
	if(!devices){
	printk("No MMC devices\n");	
	return -1;//no MMC devices in array				
	}
	
	//mmc_timer.expires = jiffies + HZ;
	//add_timer(&mmc_timer);
	//TDELAY(&mmc_timer,jiffies + HZ);
	schedule_delayed_work(&media_check,HZ);
			       
	
	
	return 0;
}

void __exit mmc_driver_exit(void)
{
	int device;
	MMCBDEBUG("mmc_driver_exit\n");
	
//	del_timer(&mmc_timer);
	
	cancel_delayed_work(&media_check);//may need a lock here
	 
	for(device=0;device<MCFMMC_MAXDEVICES;device++)
		if(mmc_devices[device])
			remove_device(global_device_ptr[device]);
		
	unregister_blkdev(MAJOR_NR, DEVICE_NAME);
}

module_init(mmc_driver_init);
module_exit(mmc_driver_exit);
