/************************************************************************
*                                                                   
*  mcf_mmc_prot.c - MMC Protocol QSPI driver for 					
*  MCF5272, MCF5235, MCF5282   										
*                                                                    
*  (C) Copyright 2005, EMAC.Inc www.emacinc.com                      
*      written by		NZG ngustavson@emacinc.com		          
*                                                                   
*																		
************************************************************************/

#define EXPORT_SYMTAB

#include <asm/coldfire.h>               /* gets us MCF_MBAR value */
#include <asm/mcfsim.h>                 /* MCFSIM offsets */
#include <asm/semaphore.h>
#include <asm/system.h>                 /* cli() and friends */
#include <asm/uaccess.h>
#include <linux/config.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/types.h>
#include <linux/version.h>
#include <linux/wait.h>
#include <linux/delay.h>


MODULE_LICENSE("Dual BSD/GPL");

/* Include versioning info, if needed */
#if (defined(MODULE) && defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS))
#define MODVERSIONS
#endif

#if defined(MODVERSIONS)
#include <linux/modversions.h>
#endif

#include <asm/mcf_qspi.h>/*for manually stealing chip selects*/
#include "mcf_mmc_prot.h"

/**
	* EXPORTED symbols
 	*/
	EXPORT_SYMBOL(MMC_Transaction);
	EXPORT_SYMBOL(MMC_BlockRead);
	EXPORT_SYMBOL(MMC_BlockWrite);
	EXPORT_SYMBOL(MMC_Init);
	EXPORT_SYMBOL(MMC_Get_Device);
	EXPORT_SYMBOL(MMC_Destroy_Device);


/**
	 * Fake CRC7. Currently the CRC is unused so
	 * hard coded to 0x95 for the first(and only used)
	 * CRC value
	 * @param array array to calculate the CRC7 on
	 * @return
	 */
static int CRC7(u8 *array){
			return 0x95;
		}
 
/**
	 * MMC packet building function
	 * This function builds the command, clears and clears the remaining
	 * frame which will be filled by the xmit function.
	 * @param frame an array object to packetize
	 * @param framelen size of the frame
	 * @param command command index number
	 * @param arg1 first arguement, high bits 31:16
	 * @param arg2 second arguement, low bits 15:0
	 * @param response number of response bits expected
	 * @return 0
	 */
	static int packetize(u8 *frame, int framelen, int command, int arg1, int arg2){
		int START_BIT = 0;//MSB is zero
		int TRANSMISSION_BIT = 0x40;
		int STOP_BIT = 0x01;				
		int bytenum;
		
		MMCDEBUG("packetize\n");
		
		frame[0] = (u8)((command|START_BIT|TRANSMISSION_BIT)&(0xff));
		frame[1] = (u8)((arg1>>8)&(0xff));
		frame[2] = (u8)((arg1)&(0xff));
		frame[3] = (u8)((arg2>>8)&(0xff));
		frame[4] = (u8)((arg2)&(0xff));
		frame[5] = (u8)(CRC7(frame)|STOP_BIT);
		
		for(bytenum=COMMAND_LENGTH;bytenum<framelen;bytenum++)
			frame[bytenum] = (u8)0xff;
	
		return 0;
	}





/**
	 * move the received response into int format
	 * @param Rnum 1,2 or 5 number of bytes received
	 * @param frame packet frame array
	 * @param bytestart index of the Rx data in the frame
	 * @return The response in int format
	 */
static int get_response(int Rnum, char *frame, int bytestart){
		int Rx = frame[bytestart];
		
		switch(Rnum){
		case 1://standard command/response
			//do nothing data alread read
		break;
		case 2://two byte response
			Rx<<=8;
			Rx|=frame[bytestart+1];
		break;
		case 5:// five byte response,currently only the READ_OCR command uses this.
			Rx = frame[bytestart+1];
			Rx<<=8;
			Rx |= frame[bytestart+2];
			Rx<<=8;
			Rx |= frame[bytestart+3];
			Rx<<=8;
			Rx |= frame[bytestart+4];
			Rx<<=8;
		break;	
			}
		return Rx;
	}	
	
/**
 * Wait functions, read a short burst of data, then if nothing has arrived yet, sleep for
 * a clock tick and try again with an eventual timeout.
 */	
	
	/**
	 * wait until the MMC returns a non-zero result
	 * @return 1
	 */
#define POLLNUMBER 100	 
	 
static int busy_wait(mmc_dev *device){
		int tickloop;
		int shortloop;
		u8 status=0;
		u8 blank = 0xff;
		device->spi->read_data.loop	= 1;
		device->spi->read_data.buf = &blank;
		device->spi->read_data.length = 1;
			
		qspi_internal_read(device->spi,&status,//add one buffer write to account for undefined (possibly ff state) 
					1,NULL,device->qcr_cs);
		
		for(tickloop=0;tickloop<100;tickloop++){
			for(shortloop=0;shortloop<POLLNUMBER;shortloop++){
				qspi_internal_read(device->spi,&status, 
					1,NULL,device->qcr_cs);
				if(status==0xff){
					qspi_internal_read(device->spi,&status, 
						1,NULL,device->qcr_cs);
					return 1;//success
				}						
			}
		set_current_state(TASK_INTERRUPTIBLE);
	 	schedule_timeout(HZ/100 ? HZ/100 : 1);
		}
			return -1;//error
	}

/**
 * delay loop that waits for an Rxtoken
 * @return 1 if successful
 */	
 
	static int wait_for_Rxtoken(mmc_dev *device) {
	u8 status=0;
	u8 blank = 0xff;
	int BR_START_TOKEN = 0xFE;//MMC token for data start
	int tickloop;
	int shortloop;
	device->spi->read_data.loop	= 1;
	device->spi->read_data.buf = &blank;
	device->spi->read_data.length = 1;		
	for(tickloop=0;tickloop<100;tickloop++){
	 	for(shortloop=0;shortloop<POLLNUMBER;shortloop++){	 		
			qspi_internal_read(device->spi,&status, 
					1,NULL,device->qcr_cs);
			if(status==(u8)BR_START_TOKEN)
				return 1;
			}
	 	set_current_state(TASK_INTERRUPTIBLE);
	 	schedule_timeout(HZ/100 ? HZ/100 : 1);
	 	}
		return -1;
		
		return 1;
}


/**
	 * interprete the command response and throw exceptions as appropriate
	 * @param response the byte containing the response
	 * @return 0 if nothing is wrong
	 */
	static int CommandResponse(int response){				
		if(response==0){
			MMCDEBUG("command successful\n");
			return 0;//no problems command succeeded
		}
		
		printk("command error\n");
		
		if((response&MMCCOM_IDLE)!=0)
			printk("idle\n");
		if((response&MMCCOM_ERASE_RES)!=0)
			printk("erase reset\n");
		if((response&MMCCOM_ILLEGAL)!=0)
			printk("illegal\n");
		if((response&MMCCOM_CRC)!=0)
			printk("crc\n");
		if((response&MMCCOM_ERASE_SEQ)!=0)
			printk("erase sequence\n");		
		if((response&MMCCOM_ADDRESS)!=0)
			printk("address\n");
		if((response&MMCCOM_PARAMETER)!=0)
			printk("parameter\n");
		
		return -1;				
	}
/**
 * interprete the data response and throw exceptions as appropriate
 * @param response the byte containing the response
 * @return 0 if nothing is wrong
 */
 	static int DataResponse(int response){
		//System.out.println("response="+response);
		
		switch(response&MMCDAT_D_RESPONSE_MASK){
		case MMCDAT_DATA_ACC:
			//System.out.println("Data accepted");
			return 0;//data was accepted, no need to complain
		case MMCDAT_CRC_ERR:
			printk("CRC error\n");
			return -1;
		case MMCDAT_WRITE_ERR:
			printk("Write error\n");
			return -1;
		default:
			printk("Communications error\n");
			return -1;
		}
		
		return -1;
	}	


/**
	 * General purpose method for tranceiving a command/response
	 * to the MMC
	 * @param command command index, see table below
	 * @param arg1 first arguement, high bits 31:16
	 * @param arg2 second arguement, low bits 15:0
	 * @param blocking block before returning?
	 * @param response number of response bits expected
	 * @return the command response
	 */
int MMC_Transaction(mmc_dev *device, int command, int arg1, int arg2,
		int blocking,int response){ 
		int Rx;
		int bytenum;
		u8 csholder=0;
		int framelen = COMMAND_LENGTH+Ncr_MAX+response;		
		unsigned char readbuff[COMMAND_LENGTH+Ncr_MAX+MAX_RESPONSE];				
		unsigned char writebuff[COMMAND_LENGTH+Ncr_MAX+MAX_RESPONSE];	
		device->spi->read_data.buf = writebuff;
		device->spi->read_data.loop	= 0;	
		device->spi->read_data.length = sizeof(writebuff);
		MMCDEBUG("transaction using cs%x ",device->qcr_cs);
				
				
		packetize(writebuff,(COMMAND_LENGTH+Ncr_MAX+response),command,arg1,arg2);
		MMCDEBUG("xmit\n");

		if(!device->inlinecall){//if called outside of another function grab cs and mutex
			qspi_mutex_down();		
			CSSTEAL(device->qcr_cs>>8,&csholder);				
		}
		
		qspi_internal_read(device->spi,readbuff,framelen,
			NULL,device->qcr_cs);
				
		MMCDEBUG("interprete\n");
		//locate and interprete the data
		for(bytenum=COMMAND_LENGTH;bytenum<framelen;bytenum++)
			if(readbuff[bytenum]!=(u8)0xff)
				break;
		
		if(bytenum==(framelen)){
			MMCDEBUG("no command response received\n");
			if(!device->inlinecall){//if called outside of another function release cs and mutex
				CSRETURN(device->spi,&csholder);
				qspi_mutex_up();
			}
			return -1;
			}
			
		MMCDEBUG("get_response\n");
		Rx = get_response(response,readbuff,bytenum);
		
		if(blocking)
			busy_wait(device);
		
		if(!device->inlinecall){//if called outside of another function release cs and mutex
		CSRETURN(device->spi,&csholder);
		qspi_mutex_up();
		}
		return Rx;
		
	}



/**
	 * Block Read transaction
	 * @param command - the initiating command
	 * @param arg1 - arg1 of the initiating command
	 * @param arg2 - arg2 of the initiating command
	 * @param response - response size 
	 * @param data - buffer to move received data into
	 * @param blocknum - number of blocks to read
	 * @param blocksize - size of blocks
	 * @param Nac - Nac timing parameter
	 * @return the total number of blocks read
	 */
int MMC_BlockRead(mmc_dev *device, int command, int arg1, int arg2,
		 int response, u8 *data,int blocksize, int blocknum,int Nac){
		u8 frame[COMMAND_LENGTH+Ncr_MAX+MAX_RESPONSE];
		int block;//current block being parsed
		int frame_index;//frame index		
		int Rx;
		u8 CRC[CRC_LENGTH];
		u8 commandbuffer[COMMAND_LENGTH];
		u8 blank = 0xff;
		u8 csholder;
		
		MMCDEBUG("BlockRead\n");			
				
		packetize(commandbuffer,COMMAND_LENGTH,command,arg1,arg2);
		
		device->spi->read_data.buf = commandbuffer;
		device->spi->read_data.loop	= 0;	
		device->spi->read_data.length = sizeof(commandbuffer);
				
		qspi_mutex_down();
		CSSTEAL(device->qcr_cs>>8,&csholder);
		
		qspi_internal_read(device->spi,frame, 
		COMMAND_LENGTH,NULL,device->qcr_cs);		
		
		device->spi->read_data.buf = &blank;
		device->spi->read_data.loop	= 1;	
		device->spi->read_data.length = 1;
		
		//new broken method cannot grab response within single xmit as it can 
		//accidentally pull data. 
		//Check the bytes as they are tranceived instead
		for(frame_index=COMMAND_LENGTH;
		frame_index<(COMMAND_LENGTH+Ncr_MAX+response);frame_index++){
						
			qspi_internal_read(device->spi,&frame[frame_index], 
			1,NULL,device->qcr_cs);	
			
			if(frame[frame_index]!=(u8)0xff)
				break;//go ahead and interprete the response
		}
			
		if(frame_index==(COMMAND_LENGTH+Ncr_MAX+response)){
			printk("MMC READ:no command response received\n");
			CSRETURN(device->spi,&csholder);
			qspi_mutex_up();
			return -1;
			}
			
		Rx = get_response(response,frame,frame_index);
		if(CommandResponse(Rx)<0){
			printk("MMC_READ:command response error\n");
			CSRETURN(device->spi,&csholder);
			qspi_mutex_up();
			return -1;//error
		}
		
		
		for(block=0;block<blocknum;block++){	
			
			if(wait_for_Rxtoken(device)<0){
				printk("MMC_READ:No Rx token received\n");
				CSRETURN(device->spi,&csholder);
				qspi_mutex_up();
				return -1;//error
			}
			//wait for Rxtoken corrupts these values, reset them
			device->spi->read_data.buf = &blank;
			device->spi->read_data.loop	= 1;	
			device->spi->read_data.length = 1;
										
			qspi_internal_read(device->spi,&data[block*blocksize], 
			blocksize,NULL,device->qcr_cs);	
			
			qspi_internal_read(device->spi,CRC, 
			CRC_LENGTH,NULL,device->qcr_cs);//not currently used	
			
			MMCDEBUG("Block %u\n",block);									
			}
					
			
			if(command==18){//if open ended read
				device->inlinecall=1;
				STOP_TRANSMISSION(device);
				device->inlinecall=0;
			}
		CSRETURN(device->spi,&csholder);
		qspi_mutex_up();		
		return blocknum;
		}
		
	
	/**
	 * Write Blocks of data;
	 * @param command command index to initiate the transfer
	 * @param arg1 first arguement to the command
	 * @param arg2 second arguement to the commmand
	 * @param response number of responses expected
	 * @param data buffer holding the data to be transmitted
	 * @param blocknum number of blocks
	 * @param blocksize size of the blocks
	 * @param Nwr timing parameter, typically 1
	 * @return
	 */
int MMC_BlockWrite(mmc_dev *device,int command, int arg1, int arg2, 
		int response, u8 *data, int blocksize, int blocknum,int Nwr){		
		int block;
		int dataresponse;
		u8 BW_STOP_TOKEN = 0xFD;	
		u8 BW_START_TOKEN = 0xFD;
		//u8 frame[COMMAND_LENGTH+Ncr_MAX+response+(blocksize+Nwr)+DATA_RESPONSE_LENGTH+CRC_LENGTH+START_TOKEN_LENGTH];
		u8 commandbuffer[COMMAND_LENGTH+MAX_NWR+MAX_RESPONSE];
		u8 responsebuffer[DATA_RESPONSE_LENGTH+CRC_LENGTH];
		u8 blank = 0xff;
		u8 csholder;
		int index;

MMCDEBUG("BlockWrite:cmd %x arg1 %x arg2 %x size %x num%x\n",command,arg1,arg2,blocksize,blocknum);					
				
//use the start correct token for single or multiple writes		
		if(command==25)
			BW_START_TOKEN = 0xFc;
		else
			BW_START_TOKEN = 0xFe;
		
//transmit command and the first and possibly only block	
		packetize(commandbuffer,sizeof(commandbuffer),command,arg1,arg2);
				
		qspi_mutex_down();
		CSSTEAL(device->qcr_cs>>8,&csholder);
	
		qspi_internal_write(device->spi,commandbuffer, 
		COMMAND_LENGTH,NULL,device->qcr_cs);	
		
		device->spi->read_data.buf = &blank;
		device->spi->read_data.loop	= 1;	
		device->spi->read_data.length = 1;
		
		for(index=COMMAND_LENGTH;
		index<(COMMAND_LENGTH+MAX_NWR+MAX_RESPONSE);index++){
						
			qspi_internal_read(device->spi,&commandbuffer[index], 
			1,NULL,device->qcr_cs);	
			
			if(commandbuffer[index]!=(u8)0xff)
				break;//go ahead and interprete the response
		}
			
		if(index==(COMMAND_LENGTH+MAX_NWR+MAX_RESPONSE)){
			printk("MMC BWRITE:no command response received\n");
			CSRETURN(device->spi,&csholder);
			qspi_mutex_up();
			return -1;
			}
			
		index = get_response(response,commandbuffer,index);
		if((CommandResponse(index))<0){
			printk("MMC BWRITE command response error\n");
			CSRETURN(device->spi,&csholder);
			qspi_mutex_up();
			return -1;//error
		}	
		
		qspi_internal_write(device->spi,&blank, 
				1,NULL,device->qcr_cs);	//NWR delay
		
		//for(block=1;block<blocknum;block++){
		for(block=0;block<blocknum;block++){
			//send start token
			qspi_internal_write(device->spi,&BW_START_TOKEN, 
				START_TOKEN_LENGTH,NULL,device->qcr_cs);	
		
			//send data
			qspi_internal_write(device->spi,&data[block*blocksize], 
				blocksize,NULL,device->qcr_cs);		

			device->spi->read_data.buf = &blank;
			device->spi->read_data.loop	= 1;	
			device->spi->read_data.length = 1;
		
			//read dataresponse
			qspi_internal_read(device->spi,responsebuffer, 
				(DATA_RESPONSE_LENGTH+CRC_LENGTH),NULL,device->qcr_cs);		


			dataresponse = responsebuffer[CRC_LENGTH];//gloss over the CRC for now	
			if(DataResponse(dataresponse)<0){
				 printk("MMC Data Response error\n");
				CSRETURN(device->spi,&csholder);
				qspi_mutex_up();
				return -1;//error
			}
			MMCDEBUG("Block %u\n",block);					
			busy_wait(device);//wait for completion
		}		

		//send the end transmit command if it's a multiple block write(open ended)
		if(command==25){
			//send stop token
			qspi_internal_write(device->spi,&BW_STOP_TOKEN, 
				1,NULL,device->qcr_cs);				
			busy_wait(device);
		}		
		
		CSRETURN(device->spi,&csholder);
		qspi_mutex_up();
		return blocknum;
	}		


 #define CARDREGTOP 15

	static int parse_CSD(CSD_register *reg, u8 *CSD){
		reg->SPEC_VERS= (CSD[CARDREGTOP-15]&0x3C)>>2;
		reg->TAAC= CSD[CARDREGTOP-14];
		reg->NSAC= CSD[CARDREGTOP-13];
		reg->TRAN_SPEED= CSD[CARDREGTOP-12];
		reg->CCC= ((CSD[CARDREGTOP-11])<<4)|((CSD[CARDREGTOP-10]&0xf0)>>4);
		reg->READ_BL_LEN= (CSD[CARDREGTOP-10]&0x0f);
		reg->READ_BL_PARTIAL= (CSD[CARDREGTOP-9]&0x80>>7);
		reg->WRITE_BLK_MISALIGN= (CSD[CARDREGTOP-9]&0x40>>6);
		reg->READ_BLK_MISALIGN= (CSD[CARDREGTOP-9]&0x20>>5);
		reg->DSR_IMP= (CSD[CARDREGTOP-9]&0x10>>4);
		reg->C_SIZE= ((CSD[CARDREGTOP-9]&0x03)<<10)|(CSD[CARDREGTOP-8]<<2)|((CSD[CARDREGTOP-7]&0xC0)>>6);		
		reg->VDD_R_CURR_MIN= (CSD[CARDREGTOP-7]&0x38)>>3;
		reg->VDD_R_CURR_MAX= (CSD[CARDREGTOP-7]&0x07);
		reg->VDD_W_CURR_MIN= (CSD[CARDREGTOP-6]&0xE0)>>5;
		reg->VDD_W_CURR_MAX= (CSD[CARDREGTOP-6]&0x1C)>>2;
		reg->C_SIZE_MULT= ((CSD[CARDREGTOP-6]&0x03)<<1)|((CSD[CARDREGTOP-5]&0x80)>>7);
		reg->ERASE_GRP_SIZE= (CSD[CARDREGTOP-5]&0x7c)>>2;
		reg->ERASE_GRP_MULT= ((CSD[CARDREGTOP-5]&0x03)<<3)|((CSD[CARDREGTOP-4]&0xe0)>>5);
		reg->WP_GRP_SIZE= (CSD[CARDREGTOP-4]&0x1f);
		reg->WP_GRP_ENABLE= (CSD[CARDREGTOP-3]&0x80)>>7;;
		reg->DEFAULT_ECC= (CSD[CARDREGTOP-3]&0x60)>>5;
		reg->R2W_FACTOR= (CSD[CARDREGTOP-3]&0x8c)>>2;
		reg->WRITE_BL_LEN= ((CSD[CARDREGTOP-3]&0x03)<<2)|((CSD[CARDREGTOP-2]&0xc0)>>6);
		reg->WRITE_BL_PARTIAL= (CSD[CARDREGTOP-2]&0x20)>>5;
		reg->FILE_FORMAT_GRP= (CSD[CARDREGTOP-1]&0x80)>>7;
		reg->COPY= (CSD[CARDREGTOP-1]&0x40)>>6;
		reg->PERM_WRITE_PROTECT= (CSD[CARDREGTOP-1]&0x20)>>5;
		reg->TMP_WRITE_PROTECT= (CSD[CARDREGTOP-1]&0x10)>>4;
		reg->FILE_FORMAT= (CSD[CARDREGTOP-1]&0x0C)>>2 ;
		reg->ECC= CSD[CARDREGTOP-1]&0x03;
		reg->CRC= CSD[CARDREGTOP-0]&0xfe;	
		return 0;
	}


static int parse_CID(CID_register *reg,u8 *CSD){
		reg->Manufacturer_ID = CSD[CARDREGTOP-15]; 
		reg->OEM_Application_ID = (CSD[CARDREGTOP-14]<<8)|(CSD[CARDREGTOP-13]); 		
		reg->Product_name[5] = CSD[CARDREGTOP-7]; 
		reg->Product_name[4] = CSD[CARDREGTOP-8]; 
		reg->Product_name[3] = CSD[CARDREGTOP-9]; 
		reg->Product_name[2] = CSD[CARDREGTOP-10]; 
		reg->Product_name[1] = CSD[CARDREGTOP-11];
		reg->Product_name[0] = CSD[CARDREGTOP-12];
		reg->Product_name[6] = 0;//NULL termination for string manipulation
		reg->Product_revision = CSD[CARDREGTOP-6];
		reg->Product_serial_number = ((long)CSD[CARDREGTOP-5]<<24)|(((long)CSD[CARDREGTOP-4]<<16)&
			0xff0000)|(((long)CSD[CARDREGTOP-3]<<8)&0xff00)|(((long)CSD[CARDREGTOP-2])&0xff); 
		reg->Manufacturing_date = CSD[CARDREGTOP-1]; 
		return 0;
	}


/**
	 * Initialize the MMC interface and the SPI interface controlling it.
	 * Also read out the registers into shadow variables.
	 */	
int MMC_Init(mmc_dev *device,int qcr_cs){
		qspi_dev *spi = device->spi;
		u8 RegisterBuff[16];
		u8 initialization[72];
		int response;
		int iteration;
		u8 csholder;
		int mult;
		
		memset(initialization,0xff,sizeof(initialization));
		
		 DDRQS = 0x7f;//set up QS lines to point out when stolen
		
		device->qcr_cs = qcr_cs<<8;
		device->blocklength = 512;/*default, change later if appropriate*/
		
        spi->bits = 8;    /* 8 bits transfers to transfer for each entry */
        spi->baud = qspi_BAUD(100000);    /* setup bit rate - 100K */
       //spi->qcd = 1;                     /* no QSPILCK delay */
        spi->qcd = 100;                     /* small QSPILCK delay */
        spi->dtl = 1;                     /* no delay after transfer */
        spi->qcr_cont = 1;      /* keep CS active throughout transfer */
        spi->odd_mod  = 0;    /* if length of buffer is a odd number, 16-bit transfers
                                           are finalized with a 8-bit transfer */
        spi->dsp_mod  = 0;    /* transfers are bounded to 15/30 bytes
                                           (= a multiple of 3 bytes = 1 word) */
        //spi->poll_mod  =  0;    /* mode polling or interrupt */
         spi->poll_mod  =  1;    /*polling */
        spi->cpol      =  0;    /* SPI clock polarity */
        spi->cpha      =  0;    /* SPI clock phase */
        spi->dohie     =  0;    /* data output high impedance disable */
		//spi->dohie     =  1;    /* data output high impedance enable */
		
		
		qspi_mutex_down();
		CSSTEAL(device->qcr_cs>>8,&csholder);			
		qspi_internal_write(spi, initialization, sizeof(initialization),
				NULL,device->qcr_cs);
		CSRETURN(device->spi,&csholder);
		qspi_mutex_up();		
		
		MMCDEBUG("GO_IDLE_STATE\n");
		GO_IDLE_STATE(device);//reset and start SPI mode
		
		MMCDEBUG("SEND_OP_COND\n");
		
		for(iteration=0;iteration<100;iteration++){
			response = SEND_OP_COND(device);//read OCR, a wait should be inserted here until the OCR indicates ready.
			MMCDEBUG("response = %u\n",response);
			if(response==0)
				break;		
		}		
		
		if(iteration==100){
			device->state=INVALID;
			return -1;
		}
		MMCDEBUG("SEND_CSD\n");
		if(SEND_CSD(device,RegisterBuff)<0){
			device->state=INVALID;
			return -1;
		}
			
		parse_CSD(&device->CSD,RegisterBuff);
		
		/*
		{//NZG test hacks
		u16 blanc = 0;
		printk("QMR = %x, dohie =%x\n",QMR,spi->dohie); 
		qspi_internal_write(spi,&blanc, 1,
				NULL,3);
		}*/
		
		MMCDEBUG("SEND_CID\n");		
		if(SEND_CID(device,RegisterBuff)<0){
			printk("MMC: no CID register found\n");
			device->state=INVALID;
			return -1;
		}
		else
			parse_CID(&device->CID,RegisterBuff);			
		
		
		
		device->blocklength = (1<<(device->CSD.READ_BL_LEN));
		device->sectors = device->CSD.C_SIZE+1;
		MMCDEBUG("set sectors to %u\n",device->sectors);
		mult = 1<<(device->CSD.C_SIZE_MULT+2);
		MMCDEBUG("mult=%u\n",mult);		
		device->sectors*=mult;
		MMCDEBUG("set sectors to %u\n",device->sectors);
		device->size = device->blocklength*device->sectors;
		device->inlinecall=0;//flag for calling functions inside functions
		
		switch(device->CSD.TRAN_SPEED&7){
			case 1:
			device->speed = 1000000;
			break;
			
			case 2:			
			device->speed = 10000000;//can't go faster than 10M
			break;
			
			case 3:			
			device->speed = MCF_CLK/2;//can't go faster than 100M(assumes Coldfire's not that fast)
			break;
			
			default:
			device->speed = 100000;
			break;
		}
		spi->baud = qspi_BAUD(device->speed);//turn the speed up to max
		
		
	printk("MMC card detected: %s\n",device->CID.Product_name);
	printk("revision: %u.%u\n",((device->CID.Product_revision&0xf0)>>4),(device->CID.Product_revision&0x0f));
	printk("serial#: %lu\n",device->CID.Product_serial_number);	
	printk("Block Length=%u\n",device->blocklength);	
	printk("Size =%lu\n",device->size);	
	printk("Speed=%lu\n",device->speed);
		
	device->state=VALID;
	return 0;											
	}


mmc_dev *MMC_Get_Device(int channel){

	qspi_dev *qspi =  qspi_create_device();
	mmc_dev *mmc;

 	if(qspi==NULL)//check that qspi device was created
				return NULL;
	
	 if ((mmc = kmalloc(sizeof(mmc_dev), GFP_KERNEL)) == NULL) {//create mmc device  
                qspi_destroy_device(qspi);
                return(NULL);
        }	
	
	mmc->spi = qspi;
	mmc->qcr_cs = channel<<8;
	spin_lock_init(&mmc->lock);
	
	//if(MMC_Init(mmc,channel)<0)
		//return MMC_Destroy_Device(mmc);//hardware not present
	
	MMC_Init(mmc,channel);//always return a device, hotplug support needs this
	
	return mmc;//return the newly created device	
}

mmc_dev *MMC_Destroy_Device(mmc_dev *dev){
	kfree(dev->spi);
	kfree(dev);
	return NULL;
}
