/************************************************************************
*                                                                     		 
*  mcf_mmc_prot.h - MMC QSPI driver  						
*  MCF5272, MCF5235, MCF5282   									
*                                                                      
*  (C) Copyright 2005, EMAC.Inc www.emacinc.com                       
*  written by		NZG ngustavson@emacinc.com		                
************************************************************************/

#if !defined(MCF_MMC_PROT_H)
#define MCF_MMC_PROT_H

#include <linux/types.h>
#include <asm/mcf_qspi.h>
#include <asm/gcfpld.h>

//packet defines
#define MAX_RESPONSE 5
#define CRC_LENGTH  2	
#define START_TOKEN_LENGTH 1
#define BR_STOP_TOKEN 0xFD
#define DATA_RESPONSE_LENGTH  1
#define MAX_NWR 8
#define COMMAND_LENGTH  6
#define Ncr_MAX  8
		
//command response defines		
#define MMCCOM_IDLE  0x01
#define MMCCOM_ERASE_RES  0x02
#define MMCCOM_ILLEGAL  0x04
#define MMCCOM_CRC  0x08
#define MMCCOM_ERASE_SEQ  0x10
#define MMCCOM_ADDRESS  0x20
#define MMCCOM_PARAMETER  0x40

//data responses
#define MMCDAT_D_RESPONSE_MASK  0x1F
#define MMCDAT_DATA_ACC  0x05
#define MMCDAT_CRC_ERR 0x0B
#define MMCDAT_WRITE_ERR 0x0D		
		


typedef struct CSD_register{
		int	SPEC_VERS;
		int	TAAC;
		int	NSAC;
		int	TRAN_SPEED;
		int	CCC;
		int	READ_BL_LEN;
		int	READ_BL_PARTIAL;
		int	WRITE_BLK_MISALIGN;
		int	READ_BLK_MISALIGN;
		int	DSR_IMP;
		int	C_SIZE;
		int	VDD_R_CURR_MIN;
		int	VDD_R_CURR_MAX;
		int	VDD_W_CURR_MIN;
		int	VDD_W_CURR_MAX;
		int	C_SIZE_MULT;
		int	ERASE_GRP_SIZE;
		int	ERASE_GRP_MULT;
		int	WP_GRP_SIZE;
		int	WP_GRP_ENABLE;
		int	DEFAULT_ECC;
		int	R2W_FACTOR;
		int	WRITE_BL_LEN;
		int	WRITE_BL_PARTIAL;
		int	FILE_FORMAT_GRP;
		int	COPY;
		int	PERM_WRITE_PROTECT;
		int	TMP_WRITE_PROTECT;
		int	FILE_FORMAT;
		int	ECC;
		int	CRC;	
	}CSD_register;	

typedef struct CID_register{
		int Manufacturer_ID; 
		int OEM_Application_ID; 
		char Product_name[7]; 
		int Product_revision;
		long Product_serial_number; 
		int Manufacturing_date; 			   							  		
	}CID_register;

typedef struct mmc_dev {
	unsigned int qcr_cs;
	unsigned int blocklength;
	unsigned int sectors;
	unsigned long size;
	unsigned long speed;
	unsigned int media_detect;
	qspi_dev *spi;	
	CSD_register CSD;
	CID_register CID;
	spinlock_t lock;
	int users;
	int inlinecall;
	int state;
	void *private_data;	
} mmc_dev;


/**
 * Prototypes for the core functions
 */
 
int MMC_Transaction(mmc_dev *device, int command, int arg1, int arg2,
		int blocking,int response);
		
int MMC_BlockRead(mmc_dev *device, int command, int arg1, int arg2,
		int response, u8 *data,int blocksize, int blocknum,int Nac);	
		
int MMC_BlockWrite(mmc_dev *device,int command, int arg1, int arg2, 
		int response, u8 *data, int blocksize, int blocknum,int Nwr);			

int MMC_Init(mmc_dev *device,int qcr_cs);

mmc_dev *MMC_Get_Device(int channel);

mmc_dev *MMC_Destroy_Device(mmc_dev *dev);


/**
 * Debug statement defines
 */
 
 //#define MMC_DEBUG 
 
#undef MMCDEBUG
#ifdef MMC_DEBUG
#define MMCDEBUG(fmt, args...) printk(fmt"\n", ## args)
#else 
#define MMCDEBUG(fmt, args...) 
#endif


/**
 * MMC Commands
 * -----------------------------------------------------------------------------
 */

/**
 * resets the MultiMedia card
 */
#define GO_IDLE_STATE(dev)	MMC_Transaction(dev,0,0,0,0,1)												
/**
 * activates the card's initialization process
 * @return command response
 */
#define	SEND_OP_COND(dev)	MMC_Transaction(dev,1,0,0,0,1)
/**
 * Asks the selected card to send its cardspecific data (CSD)
 * @param data a 16 bit array to store the CSD register in
 * @return the command response
 */
#define SEND_CSD(dev,data)  MMC_BlockRead(dev,9,0,0,1,data,16,1,8 )
/**
 * asks the selected card to send its card identification (CID)
 * @param data data a 16 bit array to store the CID register in
 * @return
 */
#define SEND_CID(dev,data)  MMC_BlockRead(dev,10,0,0,1,data,16,1, 8)
/**
 * Stop transmission on multiple block read 
 * automatically called by the READ_MULTIPLE_BLOCK function
 * @return command response
 */
#define STOP_TRANSMISSION(dev) MMC_Transaction(dev,12,0,0,0,1)
/**
 * Asks the selected card to send its status register
 * This is an untested command 
 * @return command response(in this case status)
 */
#define SEND_STATUS(dev) MMC_Transaction(dev,13,0,0,0,2)
/**
 * selects a block length (in bytes) for all following 
 * block operations (read and write)1
 * this is an untested command
 * @param arg1 top half of block length
 * @param arg2 bottom half of block length
 * @return command response
 */
#define SET_BLOCKLEN(dev,arg1,arg2) MMC_Transaction(dev,16,arg1,arg2,0,1)
/**
 * reads a block of the size selected by the SET_BLOCKLEN command2
 * changing the block size outside of the default has not been tested
 * @param address the byte address to start the read at(block aligned)
 * @param data array to move the data into
 * @param blocksize size of the blocks. If it hasn't been modified with SET_BLOCKLEN this will be BLOCKLEN
 * @return 1
 */
#define READ_SINGLE_BLOCK(dev,address,data,blocksize) MMC_BlockRead(dev,17,(int)(address>>16),(int)(address&0xffff),1, data,blocksize,1,8 )
/**
 * continuously transfers data blocks from card to host until interrupted by a 
 * stop command or the requested number of data blocks transmitted
 * Currently this command is implemented to automatically stop transmission after the 
 * number of blocks specified by blocknum is read
 * @param address starting block aligned byte address
 * @param data array to transfer the data into linearly
 * @param blocksize size of the blocks. If it hasn't been modified with 
 * SET_BLOCKLEN this will be BLOCKLEN
 * @param blocknum number of blocks to read
 * @return the number of blocks read
 */
#define READ_MULTIPLE_BLOCK(dev,address,data,blocksize,blocknum) MMC_BlockRead(dev,18,(int)(address>>16),(int)(address&0xffff),1,data,blocksize,blocknum,8)
/**
 * Defines the number of blocks which are going to be transferred in the immediately exceeding multiple block read or write command.
 * this command is untested, and probably won't work with the current read/write
 * implementation
 * @param number of blocks
 * @return command response
 */
#define SET_BLOCK_COUNT(dev,arg1) MMC_Transaction(dev,23,arg1,0,0,1)
/**
 * writes a block of the size selected by the SET_BLOCKLEN command
 * if BLOCKLEN has not been set, then the default is used (read from the CSD)
 * @param address starting block aligned byte address
 * @param data linear data array to write
 * @param blocksize size of the blocks. 
 * @return 1
 */
#define	WRITE_BLOCK(dev,address,data,blocksize)	MMC_BlockWrite(dev,24,(int)(address>>16),(int)(address&0xffff),1,data,blocksize,1,1)
/**
 * continuously writes blocks of data until a "Stop Tran" Token or the requested number of blocks received.
 * stop tran is automatically thrown by the method when the requested
 * number of blocks has been written
 * @param address address starting block aligned byte address
 * @param data linear data array to write
 * @param blocksize size of the blocks. If it hasn't been modified with SET_BLOCKLEN this will be BLOCKLEN
 * @param blocknum number of blocks to write
 * @return number of blocks written
 */
#define WRITE_MULTIPLE_BLOCK(dev,address, data, blocksize, blocknum) MMC_BlockWrite(dev,25,(int)(address>>16),(int)(address&0xffff),1,data,blocksize,blocknum,1)
/**
 * programming of the programmable bits of the CSD
 * not currently supported
 * probably a write_block method, but more research will
 * be needed.
 * @return command response
 */
#define PROGRAM_CSD(dev) MMC_Transaction(dev,27,0,0,0,1)
/**
 * if the card has write protection this command sets the write protection bit of the addressed group.
 * The properties of write protection are coded in the card specific data (WP_GRP_SIZE).
 * untested
 * @param arg1 high address bits
 * @param arg2 low address bits
 * @return command response
 */
#define SET_WRITE_PROT(dev,arg1, arg2) MMC_Transaction(dev,28,arg1,arg2,0,1)
/**
 * if the card has write protection this command clears the write protection bit of the addressed group
 * untested
 * @param arg1 high address bits
 * @param arg2 low address bits
 * @return command response
 */
#define CLR_WRITE_PROT(dev,arg1, arg2) MMC_Transaction(dev,29,arg1,arg2,true,1)
/**
 * "if the card has write protection this command asks the card to send the status of the write protection bits 
 * untested
 * @param arg1 high address bits
 * @param arg2 low address bits
 * @return command response
 */
#define SEND_WRITE_PROT(dev,arg1, arg2) MMC_Transaction(dev,30,arg1,arg2,0,1)
/**
 * sets the address of the first erase group within a range to be selected for erase
 * untested
 * @param arg1 high address bits
 * @param arg2 low address bits
 * @return command response
 */
#define TAG_ERASE_GROUP_START(dev,arg1, arg2) MMC_Transaction(dev,35,arg1,arg2,0,1)
/**
 * erases all previously selected erase groups  
 * untested
 * @param arg1 high address bits
 * @param arg2 low address bits
 * @return command response
 */
#define	ERASE(dev,arg1,arg2) MMC_Transaction(dev,38,arg1,arg2,1,1)	
/**
 * used to Set/Reset the Password or lock/unlock the card. 
 * The size of the Data Block is defined by the SET_BLOCK_LEN command.
 * untested
 * @param arg1 high address bits
 * @param arg2 low address bits
 * @return command response
 */
#define	LOCK_UNLOCK(dev,arg1,arg2) MMC_Transaction(dev,42,arg1,arg2,1,1)
/**
 * defines to the card that the next command is an application specific command 
 * rather than a standard command
 * untested
 * @param arg1 see data sheet
 * @return command response
 */
#define	APP_CMD(dev,arg1) MMC_Transaction(dev,55,arg1,0,false,1)
/**
 * used either to transfer a data block to the card or to get a data block from 
 * the card for general purpose / application specific commands. The size of the 
 * data block is defined by the SET_BLOCK_LEN command.
 * untested
 * @param arg1 high address bits
 * @param arg2 low address bits
 * @return command response
 */
#define GEN_CMD(dev,arg1, arg2)	MMC_Transaction(dev,56,arg1,arg2,true,1)
/**
 * reads the OCR register of a card 
 * untested as status bits are returned in the command response of all MMC_Transactions
 * @return command response
 */
#define READ_OCR(dev) MMC_Transaction(dev,58,0,0,0,5)
/**
 * "turns the CRC option on or off. A ?1? in the CRC option bit will turn the option on, a ?0? will turn it off"
 * CRC's are currently unsupported, turning on the CRC option will break all further MMC_Transaction until a reset
 * @param arg1 see data sheet
 * @param arg2 see data sheet
 * @return command response
 */
#define CRC_ON_OFF(dev,arg1,arg2) MMC_Transaction(dev,59,arg1,arg2,0,1)

#endif
