/***************************************************************************
                          tlv5614.c  -  SPI DtoA device function calls
                           Depends on the stackable qspi driver
    					  (C) Copyright 2005, EMAC.Inc www.emacinc.com  
                          Modified up from the pcm-37e12 version
                             -------------------
    begin                : Fri July 29 2005
    email                : ngustavson@emacinc.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         
 *   This program is free software; you can redistribute it and/or modify  
 *   it under the terms of the GNU General Public License as published by  
 *   the Free Software Foundation; either version 2 of the License, or     
 *   (at your option) any later version.                                   
 *                                                                         
 ***************************************************************************/
#include <linux/slab.h>
#include <linux/delay.h>
#include <asm/tlv5614.h>

#define ON  1
#define OFF 0

static void Select5614(tlv_dev *device, u8 status)
{
if(status==ON)
	{
	DACS_ON(device);	
	udelay(1);//need a very small delay here, couple nanos really
	LDAC_ON(device);	
	}
else
	{
	DACS_OFF(device);
	LDAC_OFF(device);
	}
}

void TLV5614_Write(tlv_dev *dev,int channel, u16 value)
{
//u16 data = ((channel<<14)|value);
u8 data[2];
data[1] = value;
data[0] = value>>8;
data[0] |= channel<<6;
//printk("TLV Write %u = %x\n",channel, *(u16 *)(data));
Select5614(dev,ON);
//qspi_internal_write(dev->spi,(u8 *)&data, 
//		sizeof(data),NULL,dev->qcr_cs);	
qspi_internal_write(dev->spi,(u8 *)data, 
		sizeof(data),NULL,dev->qcr_cs);			
Select5614(dev,OFF);
}

void TLV5614_Hardware_Init(tlv_dev *dev)
{
int chan,rep;
//printk("TLV hardware init\n");

for(rep=0;rep<2;rep++)
	for(chan=0;chan<NUMCHANNELS;chan++)
	  TLV5614_Write(dev,chan, 0);
	  
}

tlv_dev *TLV_SPIDevice(int qcr_cs){
	qspi_dev *spi;
	tlv_dev *tlv;
	
	//printk("TVL SPI device init\n");
	
	if((spi=qspi_create_device())==NULL)
		return(NULL);
	
	//printk("spi device created\n");
	
	if ((tlv = kmalloc(sizeof(tlv_dev), GFP_KERNEL)) == NULL) {//create ltc device  
                qspi_destroy_device(spi);
                return(NULL);
        }
	
	tlv->qcr_cs = qcr_cs<<8;
	
	tlv->spi = spi;
	
	memset(tlv->shadow,0,sizeof(tlv->shadow));
	
	spi->bits = 16;    /* 8 bits transfers to transfer for each entry */
	spi->baud = qspi_BAUD(10000000);    /* setup bit rate - 20M */
	spi->qcd = 1;                     /* no QSPILCK delay */
	spi->dtl = 1;                     /* no delay after transfer */
	spi->qcr_cont = 1;      /* keep CS active throughout transfer */
	spi->odd_mod  = 0;    /* if length of buffer is a odd number, 16-bit transfers are finalized with a 8-bit transfer */
	spi->dsp_mod  = 0;    /* transfers are bounded to 15/30 bytes
                                           (= a multiple of 3 bytes = 1 word) */
    spi->poll_mod  =  1;    /*polled mode */
     spi->cpol      =  1;    /* SPI clock polarity */
     spi->cpha      =  0;    /* SPI clock phase */
     spi->dohie     =  0;    /* data output high impedance enable */
     
    // TLV5614_Hardware_Init(tlv);
return tlv;
}

tlv_dev *TLV_Destroy_Device(tlv_dev *dev){
	kfree(dev->spi);
	kfree(dev);
	return NULL;
}














