/***************************************************************************
                          tlv5614_char.c  -  character driver interface to the tlv5614
    					  (C) Copyright 2005, EMAC.Inc www.emacinc.com   
                          Modified up from the pcm-37e12 version
                             -------------------
    begin                : Thur July 28 2005
    email                : ngustavson@emacinc.com

 ***************************************************************************/

/***************************************************************************
 *                                                                         
 *   This program is free software; you can redistribute it and/or modify  
 *   it under the terms of the GNU General Public License as published by  
 *   the Free Software Foundation; either version 2 of the License, or     
 *   (at your option) any later version.                                   
 *                                                                         
 ***************************************************************************/
 #include <linux/module.h>/* include file for kernel development*/
#include <linux/kernel.h>/* printk and others*/
#include <linux/fs.h>    /* the file structure */
#include <linux/ioport.h> /* inb, outb */
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/slab.h>
#include <linux/cdev.h>
#include <asm/io.h> /*ioremap*/
#include <asm/uaccess.h>

#define CHARACTER_INTERFACE
#include <asm/tlv5614.h>

#define DRIVER_VERSION "1.0"
#define DRIVER_NAME "char5614"

MODULE_AUTHOR("EMAC.Inc-NZG");
MODULE_LICENSE("Dual BSD/GPL");

//#define TLVC_DEBUG

#undef TLVCDEBUG
#ifdef TLVC_DEBUG
#define TLVCDEBUG(fmt, args...) printk(fmt"\n", ## args)
#else 
#define TLVCDEBUG(fmt, args...) 
#endif

/**
 * First Minor number for GPIO devices
 */
 #define DAQMAJOR 101
 #define FIRSTTLVMINOR 1
 #define TLVMINORNUM 1

static struct cdev *tlv5614cdev;
static tlv_dev *tlvdevice;

/**
 * TLV5614_IOCTL
 * TLV5614 control and data exchange with user space via standard ioctl call
 * additional interfaces could be added here fairly easily.
 */
static int TLV5614_IOCTL(struct inode *inode, struct file *filp, unsigned int cmd,
                unsigned long arg){

tlv_dev *dev = filp->private_data;

TLVCDEBUG("TLV5614_IOCTL\n");

switch(cmd){
	
	case READALL:{
	u16 *buffer = dev->shadow;
	int length = NUMCHANNELS*(sizeof(u16));
	if(access_ok(VERIFY_WRITE,(void *)arg,length)){
		TLVCDEBUG("copy to user %u bytes\n",length);
		copy_to_user((u16 *)arg,buffer,length);/*move data to user space*/
		return 0;
	}else{
		printk("couldn't write to user buffer\n");
		return -EINVAL;
	}}
	
	case WRITEALL:{
	int length = NUMCHANNELS*(sizeof(u16));
	int channel;
	u16 value[NUMCHANNELS];
	if(access_ok(VERIFY_READ,(void *)arg,length)){
		TLVCDEBUG("copy from user %u bytes\n",length);
		copy_from_user(value,(u16 *)arg,length);/*move data to user space*/
		for(channel=0;channel<NUMCHANNELS;channel++)
			if(value[channel]<0x1000){
				dev->shadow[channel] = value[channel];
				TLV5614_Write(dev,channel, value[channel]);
			}
		return 0;
	}else{
		printk("couldn't write to user buffer\n");
		return -EINVAL;
	}}
	
	case GETCHANNELNUM:{
		int channum = NUMCHANNELS;
		TLVCDEBUG("GETCHANNELNUM\n");
		if(access_ok(VERIFY_WRITE,(void *)arg,(sizeof(int)))){
				TLVCDEBUG("access ok\n");
				put_user(channum,(int *)arg);
				return 0;
	}else{
		printk("couldn't write to user buffer\n");
		return -EINVAL;
	}}
	
}
return -EINVAL;
}

static int TLV5614_Open (struct inode *inode, struct file *filp){	  
	  TLVCDEBUG("TLV5614_Open\n");	         
	  filp->private_data = tlvdevice;        
return 0;//don't need anything special here
}

static int TLV5614_Close(struct inode *inode, struct file *filp){
	filp->private_data  = NULL;
return 0;//don't need anything special here
}


struct file_operations TLV5614_Fops={
  .owner =    THIS_MODULE,
  .open   =    TLV5614_Open,
  .ioctl   =      TLV5614_IOCTL,
  .release =  TLV5614_Close,
};


static int TLV5614_init_module(void){
dev_t dev = MKDEV(DAQMAJOR ,FIRSTTLVMINOR);
int result;

printk(DRIVER_NAME" driver - version "DRIVER_VERSION" (c) " __DATE__  "\n");
printk(" N.Z. Gustavson (ngustavson@emacinc.com), EMAC.Inc\n");

tlvdevice =TLV5614_board_init();
if(tlvdevice==NULL){
          		printk("couldn't create tlv device\n");
                return(-ENOMEM);
        }

TLV5614_Hardware_Init(tlvdevice);

tlv5614cdev = cdev_alloc();

result = register_chrdev_region(dev, TLVMINORNUM,DRIVER_NAME);

if(result < 0){
	printk(DRIVER_NAME" driver failed to register Major Number"); 
	return result; 
	}

tlv5614cdev->ops = &TLV5614_Fops;
tlv5614cdev->owner = THIS_MODULE;
cdev_add(tlv5614cdev,dev,TLVMINORNUM);
return 0;
}

/*cleanup_module
this routine is called when a module is unloaded with rmmod
*/
static void TLV5614_cleanup_module(void){
	dev_t devno = MKDEV(DAQMAJOR ,FIRSTTLVMINOR);
	unregister_chrdev_region(devno, TLVMINORNUM);
	cdev_del(tlv5614cdev);
	TLV_Destroy_Device(tlvdevice);
}

module_init(TLV5614_init_module);
module_exit(TLV5614_cleanup_module);
